"""
Lpstat - command ``lpstat``
===========================

Parsers contains in this module are:

LpstatPrinters - command ``/usr/bin/lpstat -p``

LpstatProtocol - command ``/usr/bin/lpstat -v``
"""

from .. import parser, CommandParser
from insights.specs import Specs
from insights.parsers import SkipException

# Printer states
PRINTER_STATUS_IDLE = 'IDLE'
PRINTER_STATUS_PROCESSING = 'PROCESSING'
PRINTER_STATUS_DISABLED = 'DISABLED'
PRINTER_STATUS_UNKNOWN = 'UNKNOWN'

START_LINE_MARKER = 'printer '


@parser(Specs.lpstat_p)
class LpstatPrinters(CommandParser):
    """
    Class to parse ``lpstat -p`` command output. Parses the output of ``lpstat -p``, to get locally configured printers.
    Current available printer states are:

    * IDLE (``PRINTER_STATUS_IDLE``)
    * PROCESSING (``PRINTER_STATUS_PROCESSING``) -- printing
    * DISABLED (``PRINTER_STATUS_DISABLED``)
    * UNKNOWN (``PRINTER_STATUS_UNKNOWN``)

    Sample output of the command:

        printer idle_printer is idle.  enabled since Fri 20 Jan 2017 09:55:50 PM CET
        printer disabled_printer disabled since Wed 15 Feb 2017 12:01:11 PM EST -
        reason unknown

    Raises:
        ValueError: Raised if any error occurs parsing the content.

    Examples:
    >>> type(lpstat_printers)
    <class 'insights.parsers.lpstat.LpstatPrinters'>
    >>> len(lpstat_printers.printers)
    3
    >>> lpstat_printers.printer_names_by_status('DISABLED')
    ['disabled_printer']
    """

    def __init__(self, *args, **kwargs):
        self.printers = []
        """dict: Dictionary of locally configured printers, with keys 'name' and 'status'"""
        super(LpstatPrinters, self).__init__(*args, **kwargs)

    def parse_content(self, content):
        marker_len = len(START_LINE_MARKER)
        for line in content:
            if line.startswith(START_LINE_MARKER):
                printer_line = line[marker_len:]
                # cut printer name until next space character
                printer_name = printer_line[:printer_line.index(' ')]
                state_line_starts = marker_len + len(printer_name) + 1  # 1 is space
                state_line = line[state_line_starts:]

                printer = {
                    'name': printer_name,
                    'status': self._parse_status(state_line)
                }
                self.printers.append(printer)

    def _parse_status(self, state_line):
        if 'is idle' in state_line:
            return PRINTER_STATUS_IDLE
        elif 'printing' in state_line:
            return PRINTER_STATUS_PROCESSING
        elif 'disabled' in state_line:
            return PRINTER_STATUS_DISABLED

        return PRINTER_STATUS_UNKNOWN

    def printer_names_by_status(self, status):
        """Gives names of configured printers for a given status

        Arguments:
            status (string)
        """
        names = [prntr['name'] for prntr in self.printers if prntr['status'] == status]
        return names


@parser(Specs.lpstat_protocol_printers)
class LpstatProtocol(CommandParser, dict):
    """
    Class to parse ``lpstat -v`` command output.

    Sample output of the command::

        device for test_printer1: ipp
        device for test_printer2: ipp

    Examples:
        >>> type(lpstat_protocol)
        <class 'insights.parsers.lpstat.LpstatProtocol'>
        >>> lpstat_protocol['test_printer1']
        'ipp'
    """
    def parse_content(self, content):
        if not content:
            raise SkipException("No Valid Output")
        data = {}
        for line in content:
            if line.startswith("device for "):
                protocol = line.split(":")[-1].strip()
                printer = line.split(":")[0].split()[-1].strip()
                data[printer] = protocol
        if not data:
            raise SkipException("No Valid Output")
        self.update(data)



from insights.core.plugins import make_pass
from insights.tests import InputData, run_test

from insights.plugins import always_fires


def test_always_fires():
    i = InputData()
    expected = make_pass("ALWAYS_FIRES", kernel="this is junk")
    run_test(always_fires.report, i, expected)



"""
DockerInfo - Comand ``/usr/bin/docker info``
============================================

This parser reads the output of ``/usr/bin/docker info``.

The resulting data structure is avaible in the ``data`` member of the class
and takes the form of a dictionary whose keys are the "keys" in the output
(the string before the ``:``) and whose values are the values (the string
following the ``:``), all stripped of leading and trailing spaces.

Sample output::

    Containers: 0
    Images: 0
    Server Version: 1.9.1
    Storage Driver: devicemapper
    Pool Name: rhel-docker--pool
    Pool Blocksize: 524.3 kB
    Base Device Size: 107.4 GB
    Backing Filesystem: xfs
    Data file:
    Metadata file:
    Data Space Used: 62.39 MB
    Data Space Total: 3.876 GB
    Data Space Available: 3.813 GB
    Metadata Space Used: 40.96 kB
    Metadata Space Total: 8.389 MB
    Metadata Space Available: 8.348 MB
    Udev Sync Supported: true
    Deferred Removal Enabled: true
    Deferred Deletion Enabled: true
    Deferred Deleted Device Count: 0
    Library Version: 1.02.107-RHEL7 (2015-12-01)
    Execution Driver: native-0.2
    Logging Driver: json-file
    Kernel Version: 3.10.0-327.el7.x86_64
    CPUs: 1
    Total Memory: 993 MiB
    Name: host001.example.com
    ID: QPOX:46K6:RZK5:GPBT:DEUD:QM6H:5LRE:R63D:42DI:4BH3:6ZOZ:5EUM

Examples:

    >>> docker_info = shared[DockerInfo]
    >>> docker_info.data['Containers']
    '0'
    >>> docker_info.data['ID']
    'QPOX:46K6:RZK5:GPBT:DEUD:QM6H:5LRE:R63D:42DI:4BH3:6ZOZ:5EUM'

If the command does not return the information (for example, the Docker
daemon isn't running, the ``data`` dictionary is empty.

"""

from .. import parser, CommandParser
from insights.specs import Specs


@parser(Specs.docker_info)
class DockerInfo(CommandParser):
    """
    Represents the output of the ``/usr/bin/docker info`` command.

    The resulting output of the command is essentially key/value pairs.
    """

    def parse_content(self, content):
        self.data = {}
        # there will be more than 10 lines in the command output if "docker info"
        # command executes successfully.
        if len(content) >= 10:
            for line in content:
                if ":" in line:
                    key, value = line.strip().split(":", 1)
                    value = value.strip()
                    value = value if value else None
                    self.data[key.strip()] = value



"""
LsTmp - Command ``ls -la /tmp``
===============================

The ``ls -la /tmp`` command provides information for the listing of the
``/tmp`` directory.

"""
from insights import parser, FileListing, CommandParser
from insights.specs import Specs


@parser(Specs.ls_tmp)
class LsTmp(CommandParser, FileListing):
    """
    Parses output of ``ls -lan /tmp`` command.  See ``FileListing`` class for
    additional information.

    To prevent the output content from being too large, filters should be
    applied.  Sample output of ``ls -lan /tmp`` after filtered::

        drwxrwxr-x.  2 whuser   whuser         216 Jul  9 07:09 aws_sos
        -rw-r--r--.  1 rauser   rauser        1123 Jul 10 00:00 clean-old-archive.log
        -rw-rw-r--.  1 whuser   whuser        9620 Jul  9 07:09 daily-extraction-warehouse-run.log
        -rw-rw-r--.  1 whuser   whuser       11214 Jul  9 07:09 dask_master.log
        -rw-rw-r--.  1 whuser   whuser       27091 Jul  9 07:09 dask_worker.log
        -rw-r--r--.  1 user10   user10          29 Jul 10 00:00 date.out
        -rw-r--r--.  1 rauser   rauser   325933528 Jul 10 00:18 delete-bad-pods.log
        -rw-rw-r--.  1 whuser   whuser           0 Jul  9 07:08 extraction_driver.log
        drwxrwxrwt.  2 root     root             6 Mar 27  2017 .font-unix
        drwxrwxr-x.  3 user1    user1           17 Oct 28  2019 hadoop-user1
        drwxr-xr-x.  2 user1    user1           32 Jul  4 18:29 hsperfdata_user1
        drwxr-xr-x.  2 rauser   rauser           6 Jul 10 00:00 hsperfdata_rauser
        drwxr-xr-x.  2 root     root             6 Jul  1 14:53 hsperfdata_root
        drwxrwxrwt.  2 root     root             6 Mar 27  2017 .ICE-unix
        srw-rw----.  1 user10   user10           0 Jan  6  2020 lh_pair
        srw-rw----.  1 user10   user10           0 Jan 28 11:24 lh_pair_ex


    Examples:
        >>> type(ls_tmp)
        <class 'insights.parsers.ls_tmp.LsTmp'>
        >>> len(ls_tmp.files_of("/tmp"))
        9
        >>> len(ls_tmp.dirs_of("/tmp"))
        7
        >>> "/tmp" in ls_tmp
        True
        >>> "aws_sos" in ls_tmp.dirs_of("/tmp")
        True
        >>> "aws_sos" not in ls_tmp.files_of("/tmp")
        True
        >>> "date.out" not in ls_tmp.dirs_of("/tmp")
        True
        >>> "date.out" in ls_tmp.files_of("/tmp")
        True
    """
    pass



from insights.parsers.selinux_config import SelinuxConfig
from insights.tests import context_wrap

SELINUX_CONFIG = """
# This file controls the state of SELinux on the system.
# SELINUX= can take one of these three values:
#     enforcing - SELinux security policy is enforced.
#     permissive - SELinux prints warnings instead of enforcing.
#     disabled - No SELinux policy is loaded.
SELINUX=enforcing

 # SELINUXTYPE= can take one of three two values:
 #     targeted - Targeted processes are protected,
 #     minimum - Modification of targeted policy. Only selected processes are protected.
 #     mls - Multi Level Security protection.
SELINUXTYPE=targeted

"""


def test_selinux_config():
    selinux_config = SelinuxConfig(context_wrap(SELINUX_CONFIG)).data
    assert selinux_config["SELINUX"] == 'enforcing'
    assert selinux_config.get("SELINUXTYPE") == 'targeted'
    assert len(selinux_config) == 2



"""
datasources - Provides introspection functionality for datasources in insights-core

This module provides classes to provide introspection for each of the datasource
definition classes in the :py:mod`insights.specs` package.
"""
import inspect
import insights.core.spec_factory
import insights.specs
from insights.specs.default import DefaultSpecs as InDefaultSpecs
from insights.specs.insights_archive import InsightsArchiveSpecs as InArchiveSpecs
from insights.specs.sos_archive import SosSpecs as InSosSpecs
from . import AutologyError


SIMPLE_COMMAND_TYPE = 'simple_command'
""" str: Literal constant for a simple_comment Spec object """
SIMPLE_FILE_TYPE = 'simple_file'
""" str: Literal constant for a simple_file Spec object """
GLOB_FILE_TYPE = 'glob_file'
""" str: Literal constant for a simple_file Spec object """
FOREACH_EXECUTE_TYPE = 'foreach_execute'
""" str: Literal constant for a foreach_execute Spec object """
LISTDIR_TYPE = 'listdir'
""" str: Literal constant for a listdir Spec object """
LIST_TYPE = 'list'
""" str: Literal constant for a list Spec object """
STRING_TYPE = 'string'
""" str: Literal constant for a string Spec object """
NONE_TYPE = 'none'
""" str: Literal constant for a Spec object with None type """
FUNCTION_TYPE = 'function'
""" str: Literal constant for a function Spec object """
FIRST_FILE_TYPE = 'first_file'
""" str: Literal constant for a first_file Spec object """
FOREACH_COLLECT_TYPE = 'foreach_collect'
""" str: Literal constant for a foreach_collect Spec object """
FIRST_OF_TYPE = 'first_of'
""" str: Literal constant for a first_of Spec object """
COMMAND_WITH_ARGS_TYPE = 'command_with_args'
""" str: Literal constant for a command_with_args Spec object """
HEAD_TYPE = 'head'
""" str: Literal constant used for a head Spec object """
UNKNOWN_TYPE = 'unknown'
""" str: Literal constant for a Spec object with unknown type """
ANONYMOUS_SPEC_NAME = 'anonymous'
""" str: Literal constant used for Specs with no ``name`` attribute """


def is_simple_command(m_obj):
    """ bool: True if broker object is a simple_command object """
    return isinstance(m_obj, insights.core.spec_factory.simple_command)


def is_simple_file(m_obj):
    """ bool: True if broker object is a simple_file object """
    return isinstance(m_obj, insights.core.spec_factory.simple_file)


def is_glob_file(m_obj):
    """ bool: True if broker object is a glob_file object """
    return isinstance(m_obj, insights.core.spec_factory.glob_file)


def is_foreach_execute(m_obj):
    """ bool: True if broker object is a foreach_execute object """
    return isinstance(m_obj, insights.core.spec_factory.foreach_execute)


def is_first_file(m_obj):
    """ bool: True if broker object is a first_file object """
    return isinstance(m_obj, insights.core.spec_factory.first_file)


def is_first_of(m_obj):
    """ bool: True if broker object is a is_first_of object """
    return isinstance(m_obj, insights.core.spec_factory.first_of)


def is_foreach_collect(m_obj):
    """ bool: True if broker object is a is_foreach_collect object """
    return isinstance(m_obj, insights.core.spec_factory.foreach_collect)


def is_listdir(m_obj):
    """ bool: True if broker object is a is_listdir object """
    return isinstance(m_obj, insights.core.spec_factory.listdir)


def is_command_with_args(m_obj):
    """ bool: True if broker object is a is_command_with_args object """
    return isinstance(m_obj, insights.core.spec_factory.command_with_args)


def is_head(m_obj):
    """ bool: True if object is a head object """
    return isinstance(m_obj, insights.core.spec_factory.head)


def is_function(m_obj):
    """ bool: True if object is a function object """
    return inspect.isfunction(m_obj)


class Spec(dict):
    """
    Class to identify and describe datasources and related objects

    The normal way to create one of these objects is to use the factory
    method :py:meth:`from_object` and provide as input one of the datasource
    objects from :py:mod:`insights.core.spec_factory`.  This class is implemented
    as a dictionary and all attributes of the object are stored as dictionary keys.
    A ``repr`` string must be included in ``kwargs`` to __init__ and it will be
    removed and stored in the ``repr_str`` attribute.  If a ``name`` attributed
    is not provided, a default name of ``anonymous`` will be used.  This is a special
    name recognized when formatting Spec output to be used when specs have other
    specs as providers/dependents.

    Attributes:
        repr_str (str): String to be used to implement the __repr__ method for this object
            Supports jinja2 formatting using dictionary attributes of this object
        self (dict): All attributes of this object are included in the dictionary except
            the repr string.

    Raises:
        AutologyError: Raised if a ``repr`` string is not provided or if an unsupported
            object type is passed to the constructor or to the factory method.
    """
    def __init__(self, **kwargs):
        super(Spec, self).__init__(kwargs)
        try:
            self.repr_str = self.pop('repr')
        except KeyError:
            raise AutologyError('A repr items must be supplied for the spec name: {}'.format(self.get('name', 'no name')))

        if 'name' not in self:
            self['name'] = ANONYMOUS_SPEC_NAME

    def __repr__(self):
        try:
            formatted_str = self.repr_str.format(**self)
        except KeyError:
            if '{name} =' in self.repr_str:
                _, r = self.repr_str.split('{name} =')
                fixed = r.replace('{', '{{').replace('}', '}}')
                formatted_str = (''.join(['{name} =', fixed])).format(**self)
            else:
                formatted_str = self.repr_str.replace('{', '{{').replace('}', '}}').format(**self)
        return formatted_str

    @classmethod
    def from_object(cls, m_type, m_name=ANONYMOUS_SPEC_NAME):
        """
        Factory method to create Spec objects

        This method evaluates the m_type object type and extract the Spec
        attributes based on that type.

        Attributes:
            m_type (obj): One of the datasource objects from :py:mod:`insights.core.spec_factory`.
            m_name (str): Name of the datasource object, if not provided ``ANONYMOUS_SPEC_NAME``

        Raises:
            AutologyError: Raises this error if it cannot determine the object type

        Returns:
            Spec: Returns an object of type `Spec`
        """
        m_members = inspect.getmembers(m_type)
        m_spec = {'name': m_name, 'type': m_type}
        m_spec['kind'] = next((v for k, v in m_members if k == "kind"), None)
        m_spec['context'] = next((v for k, v in m_members if k == "context"), None)
        m_spec['multi_output'] = next((v for k, v in m_members if k == "multi_output"), None)
        if is_simple_command(m_type):
            m_spec['type_name'] = SIMPLE_COMMAND_TYPE
            m_spec['cmd'] = next((v for k, v in m_members if k == "cmd"), None)
            m_spec['repr'] = 'simple_command("{cmd}")'

        elif is_simple_file(m_type):
            m_spec['type_name'] = SIMPLE_FILE_TYPE
            m_spec['path'] = next((v for k, v in m_members if k == "path"), None)
            m_spec['repr'] = 'simple_file("{path}")'

        elif is_glob_file(m_type):
            m_spec['type_name'] = GLOB_FILE_TYPE
            m_spec['patterns'] = next((v for k, v in m_members if k == "patterns"), None)
            m_spec['repr'] = 'glob_file({patterns})'

        elif is_first_file(m_type):
            m_spec['type_name'] = FIRST_FILE_TYPE
            m_spec['paths'] = next((v for k, v in m_members if k == "paths"), None)
            m_spec['repr'] = 'first_file({paths})'

        elif is_listdir(m_type):
            m_spec['type_name'] = LISTDIR_TYPE
            m_spec['path'] = next((v for k, v in m_members if k == 'path'), None)
            m_spec['repr'] = 'listdir("{path}")'

        elif is_foreach_execute(m_type):
            m_spec['cmd'] = next((v for k, v in m_members if k == "cmd"), None)
            m_spec['type_name'] = FOREACH_EXECUTE_TYPE
            provider = next((v for k, v in m_members if k == "provider"), None)
            if provider:
                m_spec['provider'] = cls.from_object(provider)

            else:
                m_spec['provider'] = Spec(
                    name=ANONYMOUS_SPEC_NAME,
                    type=None,
                    type_name=NONE_TYPE,
                    repr='NONE PROVIDER')

            m_spec['repr'] = 'foreach_execute("{cmd}", provider={provider})'

        elif is_first_of(m_type):
            m_spec['type_name'] = FIRST_OF_TYPE
            deps = next((v for k, v in m_members if k == "deps"), None)
            m_spec['deps'] = [cls.from_object(d) for d in deps]
            deps_repr = ', '.join(['{0}'.format(d) for d in m_spec['deps']])
            m_spec['repr'] = 'first_of([{0}])'.format(deps_repr)

        elif is_command_with_args(m_type):
            m_spec['type_name'] = COMMAND_WITH_ARGS_TYPE
            m_spec['cmd'] = next((v for k, v in m_members if k == "cmd"), None)
            provider = next((v for k, v in m_members if k == "provider"), None)
            if provider:
                m_spec['provider'] = cls.from_object(provider)
            m_spec['repr'] = 'command_with_args("{cmd}", provider={provider})'

        elif is_foreach_collect(m_type):
            m_spec['type_name'] = FOREACH_COLLECT_TYPE
            m_spec['path'] = next((v for k, v in m_members if k == 'path'), None)
            provider = next((v for k, v in m_members if k == "provider"), None)
            if provider:
                m_spec['provider'] = cls.from_object(provider)
            m_spec['repr'] = 'foreach_collect("{path}", provider={provider})'

        elif is_head(m_type):
            m_spec['type_name'] = HEAD_TYPE
            dep = next((v for k, v in m_members if k == "dep"), None)
            m_spec['dep'] = cls.from_object(dep)
            m_spec['repr'] = 'head({0})'.format(m_spec['dep'])

        elif m_type is None:
            m_spec['type_name'] = NONE_TYPE
            m_spec['repr'] = 'NONE TYPE'

        elif inspect.isfunction(m_type):
            m_spec['type_name'] = FUNCTION_TYPE
            f_members = inspect.getmembers(m_type)
            m_spec['fxn_name'] = next((v for k, v in f_members if k == '__name__'), 'function')
            m_spec['source'] = inspect.getsource(m_type)
            m_spec['repr'] = '{fxn_name}()'

        elif isinstance(m_type, list):
            m_spec['type_name'] = LIST_TYPE
            m_spec['list'] = m_type
            m_spec['repr'] = '{list}'

        elif isinstance(m_type, str):
            m_spec['type_name'] = STRING_TYPE
            m_spec['string'] = m_type
            m_spec['repr'] = '{string}'

        else:
            raise AutologyError('Unsupported name {} object {}, please add it'.format(m_name, m_type))

        m_spec['repr'] = ''.join(['{name} = ', m_spec['repr']]) if m_name != ANONYMOUS_SPEC_NAME else m_spec['repr']

        return cls(**m_spec)

    @property
    def is_simple_command(self):
        """ bool: True if this spec is a simple_command """
        return self.get('type_name', UNKNOWN_TYPE) == SIMPLE_COMMAND_TYPE

    @property
    def is_simple_file(self):
        """ bool: True if this spec is a simple_file """
        return self.get('type_name', UNKNOWN_TYPE) == SIMPLE_FILE_TYPE

    @property
    def is_glob_file(self):
        """ bool: True if this spec is a glob_file """
        return self.get('type_name', UNKNOWN_TYPE) == GLOB_FILE_TYPE

    @property
    def is_foreach_execute(self):
        """ bool: True if this spec is a foreach_execute """
        return self.get('type_name', UNKNOWN_TYPE) == FOREACH_EXECUTE_TYPE

    @property
    def is_first_file(self):
        """ bool: True if this spec is a first_file """
        return self.get('type_name', UNKNOWN_TYPE) == FIRST_FILE_TYPE

    @property
    def is_first_of(self):
        """ bool: True if this spec is a first_of """
        return self.get('type_name', UNKNOWN_TYPE) == FIRST_OF_TYPE

    @property
    def is_foreach_collect(self):
        """ bool: True if this spec is a foreach_collect """
        return self.get('type_name', UNKNOWN_TYPE) == FOREACH_COLLECT_TYPE

    @property
    def is_listdir(self):
        """ bool: True if this spec is a listdir """
        return self.get('type_name', UNKNOWN_TYPE) == LISTDIR_TYPE

    @property
    def is_command_with_args(self):
        """ bool: True if this spec is a command_with_args """
        return self.get('type_name', UNKNOWN_TYPE) == COMMAND_WITH_ARGS_TYPE

    @property
    def is_head(self):
        """ bool: True if this spec is a head """
        return self.get('type_name', UNKNOWN_TYPE) == HEAD_TYPE

    @property
    def is_function(self):
        """ bool: True if this spec is a function """
        return self.get('type_name', UNKNOWN_TYPE) == FUNCTION_TYPE

    @property
    def is_unknown_type(self):
        """ bool: True if this spec has an unknown type """
        return self.get('type_name', UNKNOWN_TYPE) == UNKNOWN_TYPE


class RegistrySpecs(dict):
    """
    Class to provide introspection for Registry objects in :py:class:`insights.specs.Specs`

    Dictionary of `Spec` objects with spec names as the keys and a Spec object as each value.
    Each value has the set of attributes::

        name - Name of the spec
        type - The spec object
        filterable - Whether or not the Registry object is ``filterable``
        multi_output - Whether or not the Registry object is ``multi_output``
        raw - Whether or not the Registry object is ``raw``
    """
    REGISTRY_REPR = '{name} = RegistryPoint(filterable={filterable}, multi_output={multi_output}, raw={raw})'
    """ str: repr string for all Registry objects """

    def __init__(self):
        members = inspect.getmembers(insights.specs.Specs)
        for m_name, m_type in members:
            if m_name.startswith('__') or m_name == 'context_handlers':
                # Don't care about dunder members or context handlers
                continue

            m_members = inspect.getmembers(m_type)
            self.update({
                m_name: Spec(
                    name=m_name,
                    type=m_type,
                    filterable=next((v for k, v in m_members if k == 'filterable'), None),
                    multi_output=next((v for k, v in m_members if k == 'multi_output'), None),
                    raw=next((v for k, v in m_members if k == 'raw'), None),
                    repr=self.REGISTRY_REPR
                )
            })

    def is_registered(self, spec_name):
        """ bool: Whether ``spec_name`` is in Registry specs. """
        return spec_name in self


class DefaultSpecs(dict):
    """
    Class to provide introspection for datasource objects in :py:class:`insights.specs.default.DefaultSpecs`

    Dictionary of `Spec` objects with spec names as the keys and a Spec object as each value.
    Each Spec has different attributes depending on the type of spec.  See the
    :py:meth:`Spec.from_object` factory method for more information.

    """
    _SPECS_OBJECT_CLASS = InDefaultSpecs
    """ obj: Datasource class to use for introspection """

    def __init__(self):
        for m_name, m_type in inspect.getmembers(self._SPECS_OBJECT_CLASS):
            if (m_name.startswith('__') or m_name == 'context_handlers' or
                    m_name == 'registry' or str(m_type).startswith('insights.specs.Spec')):
                # Don't care about dunder members or context handlers
                # or registry entries
                continue

            m_spec = Spec.from_object(m_type, m_name)
            self.update({m_name: m_spec})


class InsightsArchiveSpecs(DefaultSpecs):
    """
    Class to provide introspection for datasource objects in
    :py:class:`insights.specs.insights_archive.InsightsArchiveSpecs`

    Dictionary of `Spec` objects with spec names as the keys and a Spec object as each value.
    Each Spec has different attributes depending on the type of spec.  See the
    :py:meth:`Spec.from_object` factory method for more information.

    """
    _SPECS_OBJECT_CLASS = InArchiveSpecs
    """ obj: Datasource class to use for introspection """

    pass


class SosSpecs(DefaultSpecs):
    """
    Class to provide introspection for datasource objects in :py:class:`insights.specs.sos_archive.SosSpecs`

    Dictionary of `Spec` objects with spec names as the keys and a Spec object as each value.
    Each Spec has different attributes depending on the type of spec.  See the
    :py:meth:`Spec.from_object` factory method for more information.

    """
    _SPECS_OBJECT_CLASS = InSosSpecs
    """ obj: Datasource class to use for introspection """

    pass


if __name__ == "__main__":
    specs = DefaultSpecs()
    for k, v in specs.items():
        try:
            print(v)
        except Exception:
            print('======= Error with spec: ', k)
            print('repr_str :', v.repr_str)
            for dk, dv in v.items():
                print('\t', dk, ":", dv)
            raise



"""
LsVarWwwPerms - command ``/bin/ls -la /dev/null /var/www``
==========================================================
"""

from insights.core import CommandParser, FileListing
from insights.core.plugins import parser
from insights.specs import Specs
from insights.util.file_permissions import FilePermissions


@parser(Specs.ls_var_www)
class LsVarWwwPerms(CommandParser, FileListing):
    """
    Class for parsing ``/bin/ls -la /dev/null /var/www`` command.

    Attributes:
        file_permissions (list): list of `FilePermissions` objects for every file from the output

    Sample output of this command is::

        crw-rw-rw-. 1 root root 1, 3 Dec 18 09:18 /dev/null

        /var/www:
        total 16
        drwxr-xr-x.  4 root root  33 Dec 15 08:12 .
        drwxr-xr-x. 20 root root 278 Dec 15 08:12 ..
        drwxr-xr-x.  2 root root   6 Oct  3 09:37 cgi-bin
        drwxr-xr-x.  2 root root   6 Oct  3 09:37 html

    Examples:

        >>> type(ls_var_www_perms)
        <class 'insights.parsers.ls_var_www_perms.LsVarWwwPerms'>
        >>> ls_var_www_perms.file_permissions[2]
        FilePermissions(cgi-bin)
        >>> ls_var_www_perms.file_permissions[2].line
        'drwxr-xr-x.  2 root root   6 Oct  3 09:37 cgi-bin'
    """

    def parse_content(self, content):
        super(LsVarWwwPerms, self).parse_content(content)

        self.file_permissions = []
        if "/var/www" in self:
            for filename, info in sorted(self.listing_of("/var/www").items()):
                self.file_permissions.append(FilePermissions(info["raw_entry"]))



from insights.core.context import OSP
from insights.parsers.cinder_conf import CinderConf
from insights.tests import context_wrap

cinder_content = """
[DEFAULT]

#
# Options defined in oslo.messaging
#

# ZeroMQ bind address. Should be a wildcard (*), an ethernet
# interface, or IP. The "host" option should point or resolve
# to this address. (string value)
#rpc_zmq_bind_address=*

# MatchMaker driver. (string value)
#rpc_zmq_matchmaker=local

# ZeroMQ receiver listening port. (integer value)
#rpc_zmq_port=9501

# Number of ZeroMQ contexts, defaults to 1. (integer value)
#rpc_zmq_contexts=1

# Maximum number of ingress messages to locally buffer per
# topic. Default is unlimited. (integer value)
#rpc_zmq_topic_backlog=<None>

# Directory for holding IPC sockets. (string value)
#rpc_zmq_ipc_dir=/var/run/openstack

# Name of this node. Must be a valid hostname, FQDN, or IP
# address. Must match "host" option, if running Nova. (string
# value)
#rpc_zmq_host=cinder

# Seconds to wait before a cast expires (TTL). Only supported
# by impl_zmq. (integer value)
#rpc_cast_timeout=30

# Heartbeat frequency. (integer value)
#matchmaker_heartbeat_freq=300

# Heartbeat time-to-live. (integer value)
#matchmaker_heartbeat_ttl=600

# Size of RPC thread pool. (integer value)
#rpc_thread_pool_size=64

# Driver or drivers to handle sending notifications. (multi
# valued)
#notification_driver=

# AMQP topic used for OpenStack notifications. (list value)
# Deprecated group/name - [rpc_notifier2]/topics
#notification_topics=notifications

# Seconds to wait for a response from a call. (integer value)
#rpc_response_timeout=60

# A URL representing the messaging driver to use and its full
# configuration. If not set, we fall back to the rpc_backend
# option and driver specific configuration. (string value)
#transport_url=<None>

# The messaging driver to use, defaults to rabbit. Other
# drivers include qpid and zmq. (string value)
#rpc_backend=rabbit
rpc_backend=cinder.openstack.common.rpc.impl_kombu

# The default exchange under which topics are scoped. May be
# overridden by an exchange name specified in the
# transport_url option. (string value)
#control_exchange=openstack
control_exchange=openstack


#
# Options defined in cinder.exception
#

# Make exception message format errors fatal. (boolean value)
#fatal_exception_format_errors=false


#
# Options defined in cinder.quota
#

# Number of volumes allowed per project (integer value)
#quota_volumes=10

# Number of volume snapshots allowed per project (integer
# value)
#quota_snapshots=10

# Number of consistencygroups allowed per project (integer
# value)
#quota_consistencygroups=10

# Total amount of storage, in gigabytes, allowed for volumes
# and snapshots per project (integer value)
#quota_gigabytes=1000

# Number of volume backups allowed per project (integer value)
#quota_backups=10

# Total amount of storage, in gigabytes, allowed for backups
# per project (integer value)
#quota_backup_gigabytes=1000

# Number of seconds until a reservation expires (integer
# value)
#reservation_expire=86400

# Count of reservations until usage is refreshed (integer
# value)
#until_refresh=0

# Number of seconds between subsequent usage refreshes
# (integer value)
#max_age=0

# Default driver to use for quota checks (string value)
#quota_driver=cinder.quota.DbQuotaDriver

# Enables or disables use of default quota class with default
# quota. (boolean value)
#use_default_quota_class=true


#
# Options defined in cinder.service
#

# Interval, in seconds, between nodes reporting state to
# datastore (integer value)
#report_interval=10

# Interval, in seconds, between running periodic tasks
# (integer value)
#periodic_interval=60

# Range, in seconds, to randomly delay when starting the
# periodic task scheduler to reduce stampeding. (Disable by
# setting to 0) (integer value)
#periodic_fuzzy_delay=60

# IP address on which OpenStack Volume API listens (string
# value)
#osapi_volume_listen=0.0.0.0
osapi_volume_listen=10.22.100.58

# Port on which OpenStack Volume API listens (integer value)
#osapi_volume_listen_port=8776

# Number of workers for OpenStack Volume API service. The
# default is equal to the number of CPUs available. (integer
# value)
#osapi_volume_workers=<None>
osapi_volume_workers=32


#
# Options defined in cinder.ssh_utils
#

# Option to enable strict host key checking.  When set to
# "True" Cinder will only connect to systems with a host key
# present in the configured "ssh_hosts_key_file".  When set to
# "False" the host key will be saved upon first connection and
# used for subsequent connections.  Default=False (boolean
# value)
#strict_ssh_host_key_policy=false

# File containing SSH host keys for the systems with which
# Cinder needs to communicate.  OPTIONAL:
# Default=$state_path/ssh_known_hosts (string value)
#ssh_hosts_key_file=$state_path/ssh_known_hosts
#
# Options defined in cinder.test
#

# File name of clean sqlite db (string value)
#sqlite_clean_db=clean.sqlite


#
# Options defined in cinder.wsgi
#

# Maximum line size of message headers to be accepted.
# max_header_line may need to be increased when using large
# tokens (typically those generated by the Keystone v3 API
# with big service catalogs). (integer value)
#max_header_line=16384

# Timeout for client connections' socket operations. If an
# incoming connection is idle for this number of seconds it
# will be closed. A value of '0' means wait forever. (integer
# value)
#client_socket_timeout=900

# If False, closes the client socket connection explicitly.
# Setting it to True to maintain backward compatibility.
# Recommended setting is set it to False. (boolean value)
#wsgi_keep_alive=true

# Sets the value of TCP_KEEPALIVE (True/False) for each server
# socket. (boolean value)
#tcp_keepalive=true

# Sets the value of TCP_KEEPIDLE in seconds for each server
# socket. Not supported on OS X. (integer value)
#tcp_keepidle=600

# Sets the value of TCP_KEEPINTVL in seconds for each server
# socket. Not supported on OS X. (integer value)
#tcp_keepalive_interval=<None>

# Sets the value of TCP_KEEPCNT for each server socket. Not
# supported on OS X. (integer value)
#tcp_keepalive_count=<None>

# CA certificate file to use to verify connecting clients
# (string value)
#ssl_ca_file=<None>

# Certificate file to use when starting the server securely
# (string value)
#ssl_cert_file=<None>

# Private key file to use when starting the server securely
# (string value)
#ssl_key_file=<None>


#
# Options defined in cinder.api.common
#

# The maximum number of items that a collection resource
# returns in a single response (integer value)
#osapi_max_limit=1000

# Base URL that will be presented to users in links to the
# OpenStack Volume API (string value)
# Deprecated group/name - [DEFAULT]/osapi_compute_link_prefix
#osapi_volume_base_URL=<None>


#
# Options defined in cinder.api.middleware.auth
#

# Treat X-Forwarded-For as the canonical remote address. Only
# enable this if you have a sanitizing proxy. (boolean value)
#use_forwarded_for=false


#
# Options defined in cinder.api.middleware.sizelimit
#

# Max size for body of a request (integer value)
#osapi_max_request_body_size=114688


#
# Options defined in cinder.api.views.versions
#

# Public url to use for versions endpoint. The default is
# None, which will use the request's host_url attribute to
# populate the URL base. If Cinder is operating behind a
# proxy, you will want to change this to represent the proxy's
# URL. (string value)
#public_endpoint=<None>


#
# Options defined in cinder.backup.chunkeddriver
#

# Compression algorithm (None to disable) (string value)
#backup_compression_algorithm=zlib


#
# Options defined in cinder.backup.driver
#

# Backup metadata version to be used when backing up volume
# metadata. If this number is bumped, make sure the service
# doing the restore supports the new version. (integer value)
#backup_metadata_version=2

# The number of chunks or objects, for which one Ceilometer
# notification will be sent (integer value)
#backup_object_number_per_notification=10

# Interval, in seconds, between two progress notifications
# reporting the backup status (integer value)
#backup_timer_interval=120


#
# Options defined in cinder.backup.drivers.ceph
#

# Ceph configuration file to use. (string value)
#backup_ceph_conf=/etc/ceph/ceph.conf

# The Ceph user to connect with. Default here is to use the
# same user as for Cinder volumes. If not using cephx this
# should be set to None. (string value)
#backup_ceph_user=cinder

# The chunk size, in bytes, that a backup is broken into
# before transfer to the Ceph object store. (integer value)
#backup_ceph_chunk_size=134217728

# The Ceph pool where volume backups are stored. (string
# value)
#backup_ceph_pool=backups

# RBD stripe unit to use when creating a backup image.
# (integer value)
#backup_ceph_stripe_unit=0

# RBD stripe count to use when creating a backup image.
# (integer value)
#backup_ceph_stripe_count=0

# If True, always discard excess bytes when restoring volumes
# i.e. pad with zeroes. (boolean value)
#restore_discard_excess_bytes=true


#
# Options defined in cinder.backup.drivers.nfs
#

# The maximum size in bytes of the files used to hold backups.
# If the volume being backed up exceeds this size, then it
# will be backed up into multiple files. (integer value)
#backup_file_size=1999994880

# The size in bytes that changes are tracked for incremental
# backups. backup_swift_object_size has to be multiple of
# backup_swift_block_size. (integer value)
#backup_sha_block_size_bytes=32768

# Enable or Disable the timer to send the periodic progress
# notifications to Ceilometer when backing up the volume to
# the backend storage. The default value is True to enable the
# timer. (boolean value)
#backup_enable_progress_timer=true

# Base dir containing mount point for NFS share. (string
# value)
#backup_mount_point_base=$state_path/backup_mount

# NFS share in fqdn:path, ipv4addr:path, or "[ipv6addr]:path"
# format. (string value)
#backup_share=<None>

# Mount options passed to the NFS client. See NFS man page for
# details. (string value)
#backup_mount_options=<None>

# Custom container to use for backups. (string value)
#backup_container=<None>


#
# Options defined in cinder.backup.drivers.swift
#

# The URL of the Swift endpoint (string value)
#backup_swift_url=<None>

# Info to match when looking for swift in the service catalog.
# Format is: separated values of the form:
# <service_type>:<service_name>:<endpoint_type> - Only used if
# backup_swift_url is unset (string value)
#swift_catalog_info=object-store:swift:publicURL

# Swift authentication mechanism (string value)
#backup_swift_auth=per_user

# Swift authentication version. Specify "1" for auth 1.0, or
# "2" for auth 2.0 (string value)
#backup_swift_auth_version=1

# Swift tenant/account name. Required when connecting to an
# auth 2.0 system (string value)
#backup_swift_tenant=<None>

# Swift user name (string value)
#backup_swift_user=<None>

# Swift key for authentication (string value)
#backup_swift_key=<None>

# The default Swift container to use (string value)
#backup_swift_container=volumebackups

# The size in bytes of Swift backup objects (integer value)
#backup_swift_object_size=52428800

# The size in bytes that changes are tracked for incremental
# backups. backup_swift_object_size has to be multiple of
# backup_swift_block_size. (integer value)
#backup_swift_block_size=32768

# The number of retries to make for Swift operations (integer
# value)
#backup_swift_retry_attempts=3

# The backoff time in seconds between Swift retries (integer
# value)
#backup_swift_retry_backoff=2

# Enable or Disable the timer to send the periodic progress
# notifications to Ceilometer when backing up the volume to
# the Swift backend storage. The default value is True to
# enable the timer. (boolean value)
#backup_swift_enable_progress_timer=true


#
# Options defined in cinder.backup.drivers.tsm
#

# Volume prefix for the backup id when backing up to TSM
# (string value)
#backup_tsm_volume_prefix=backup

# TSM password for the running username (string value)
#backup_tsm_password=password

# Enable or Disable compression for backups (boolean value)
#backup_tsm_compression=true


#
# Options defined in cinder.backup.manager
#

# Driver to use for backups. (string value)
# Deprecated group/name - [DEFAULT]/backup_service
#backup_driver=cinder.backup.drivers.swift


#
# Options defined in cinder.cmd.volume
#

# Backend override of host value. (string value)
# Deprecated group/name - [DEFAULT]/host
#backend_host=<None>


#
# Options defined in cinder.cmd.volume_usage_audit
#

# If this option is specified then the start time specified is
# used instead of the start time of the last completed audit
# period. (string value)
#start_time=<None>

# If this option is specified then the end time specified is
# used instead of the end time of the last completed audit
# period. (string value)
#end_time=<None>

# Send the volume and snapshot create and delete notifications
# generated in the specified period. (boolean value)
#send_actions=false


#
# Options defined in cinder.common.config
#

# File name for the paste.deploy config for cinder-api (string
# value)
#api_paste_config=api-paste.ini
api_paste_config=/etc/cinder/api-paste.ini

# Top-level directory for maintaining cinder's state (string
# value)
# Deprecated group/name - [DEFAULT]/pybasedir
#state_path=/var/lib/cinder

# IP address of this host (string value)
#my_ip=10.0.0.1

# Default glance host name or IP (string value)
#glance_host=$my_ip

# Default glance port (integer value)
#glance_port=9292

# A list of the glance API servers available to cinder
# ([hostname|ip]:port) (list value)
#glance_api_servers=$glance_host:$glance_port
glance_api_servers=http://10.22.120.50:9292

# Version of the glance API to use (integer value)
#glance_api_version=1
glance_api_version=2

# Number retries when downloading an image from glance
# (integer value)
#glance_num_retries=0
glance_num_retries=0

# Allow to perform insecure SSL (https) requests to glance
# (boolean value)
#glance_api_insecure=false
glance_api_insecure=False

# Enables or disables negotiation of SSL layer compression. In
# some cases disabling compression can improve data
# throughput, such as when high network bandwidth is available
# and you use compressed image formats like qcow2. (boolean
# value)
#glance_api_ssl_compression=false
glance_api_ssl_compression=False

# Location of ca certificates file to use for glance client
# requests. (string value)
#glance_ca_certificates_file=<None>

# http/https timeout value for glance operations. If no value
# (None) is supplied here, the glanceclient default value is
# used. (integer value)
#glance_request_timeout=<None>

# The topic that scheduler nodes listen on (string value)
#scheduler_topic=cinder-scheduler

# The topic that volume nodes listen on (string value)
#volume_topic=cinder-volume

# The topic that volume backup nodes listen on (string value)
#backup_topic=cinder-backup

# DEPRECATED: Deploy v1 of the Cinder API. (boolean value)
#enable_v1_api=true
enable_v1_api=True

# Deploy v2 of the Cinder API. (boolean value)
#enable_v2_api=true
enable_v2_api=True

# Enables or disables rate limit of the API. (boolean value)
#api_rate_limit=true

# Specify list of extensions to load when using
# osapi_volume_extension option with
# cinder.api.contrib.select_extensions (list value)
#osapi_volume_ext_list=

# osapi volume extension to load (multi valued)
#osapi_volume_extension=cinder.api.contrib.standard_extensions

# Full class name for the Manager for volume (string value)
#volume_manager=cinder.volume.manager.VolumeManager

# Full class name for the Manager for volume backup (string
# value)
#backup_manager=cinder.backup.manager.BackupManager

# Full class name for the Manager for scheduler (string value)
#scheduler_manager=cinder.scheduler.manager.SchedulerManager

# Name of this node.  This can be an opaque identifier. It is
# not necessarily a host name, FQDN, or IP address. (string
# value)
#host=cinder

# Availability zone of this node (string value)
#storage_availability_zone=nova
storage_availability_zone=nova

# Default availability zone for new volumes. If not set, the
# storage_availability_zone option value is used as the
# default for new volumes. (string value)
#default_availability_zone=<None>
default_availability_zone=nova

# Default volume type to use (string value)
#default_volume_type=<None>

# Time period for which to generate volume usages. The options
# are hour, day, month, or year. (string value)
#volume_usage_audit_period=month

# Path to the rootwrap configuration file to use for running
# commands as root (string value)
#rootwrap_config=/etc/cinder/rootwrap.conf

# Enable monkey patching (boolean value)
#monkey_patch=false

# List of modules/decorators to monkey patch (list value)
#monkey_patch_modules=

# Maximum time since last check-in for a service to be
# considered up (integer value)
#service_down_time=60

# The full class name of the volume API class to use (string
# value)
#volume_api_class=cinder.volume.api.API

# The full class name of the volume backup API class (string
# value)
#backup_api_class=cinder.backup.api.API

# The strategy to use for auth. Supports noauth, keystone, and
# deprecated. (string value)
#auth_strategy=noauth
auth_strategy=keystone

# A list of backend names to use. These backend names should
# be backed by a unique [CONFIG] group with its options (list
# value)
#enabled_backends=<None>
enabled_backends=tripleo_ceph

# Whether snapshots count against gigabyte quota (boolean
# value)
#no_snapshot_gb_quota=false

# The full class name of the volume transfer API class (string
# value)
#transfer_api_class=cinder.transfer.api.API

# The full class name of the volume replication API class
# (string value)
#replication_api_class=cinder.replication.api.API

# The full class name of the consistencygroup API class
# (string value)
#consistencygroup_api_class=cinder.consistencygroup.api.API

# OpenStack privileged account username. Used for requests to
# other services (such as Nova) that require an account with
# special rights. (string value)
#os_privileged_user_name=<None>

# Password associated with the OpenStack privileged account.
# (string value)
#os_privileged_user_password=<None>

# Tenant name associated with the OpenStack privileged
# account. (string value)
#os_privileged_user_tenant=<None>


#
# Options defined in cinder.compute
#

# The full class name of the compute API class to use (string
# value)
#compute_api_class=cinder.compute.nova.API


#
# Options defined in cinder.compute.nova
#

# Match this value when searching for nova in the service
# catalog. Format is: separated values of the form:
# <service_type>:<service_name>:<endpoint_type> (string value)
#nova_catalog_info=compute:Compute Service:publicURL
nova_catalog_info=compute:Compute Service:publicURL

# Same as nova_catalog_info, but for admin endpoint. (string
# value)
#nova_catalog_admin_info=compute:Compute Service:adminURL
nova_catalog_admin_info=compute:Compute Service:adminURL

# Override service catalog lookup with template for nova
# endpoint e.g. http://localhost:8774/v2/%(project_id)s
# (string value)
#nova_endpoint_template=<None>

# Same as nova_endpoint_template, but for admin endpoint.
# (string value)
#nova_endpoint_admin_template=<None>

# Region name of this node (string value)
#os_region_name=<None>

# Location of ca certificates file to use for nova client
# requests. (string value)
#nova_ca_certificates_file=<None>

# Allow to perform insecure SSL requests to nova (boolean
# value)
#nova_api_insecure=false


#
# Options defined in cinder.db.api
#

# Services to be added to the available pool on create
# (boolean value)
#enable_new_services=true

# Template string to be used to generate volume names (string
# value)
#volume_name_template=volume-%s

# Template string to be used to generate snapshot names
# (string value)
#snapshot_name_template=snapshot-%s

# Template string to be used to generate backup names (string
# value)
#backup_name_template=backup-%s


#
# Options defined in cinder.db.base
#

# Driver to use for database access (string value)
#db_driver=cinder.db
#
# Options defined in cinder.openstack.common.periodic_task
#

# Some periodic tasks can be run in a separate process. Should
# we run them here? (boolean value)
#run_external_periodic_tasks=true


#
# Options defined in cinder.openstack.common.policy
#

# The JSON file that defines policies. (string value)
#policy_file=policy.json

# Default rule. Enforced when a requested rule is not found.
# (string value)
#policy_default_rule=default

# Directories where policy configuration files are stored.
# They can be relative to any directory in the search path
# defined by the config_dir option, or absolute paths. The
# file defined by policy_file must exist for these directories
# to be searched.  Missing or empty directories are ignored.
# (multi valued)
#policy_dirs=policy.d
# Which filter class names to use for filtering hosts when not
# specified in the request. (list value)
#scheduler_default_filters=AvailabilityZoneFilter,CapacityFilter,CapabilitiesFilter

# Which weigher class names to use for weighing hosts. (list
# value)

# Cache volume availability zones in memory for the provided
# duration in seconds (integer value)
#az_cache_duration=3600

# Create volume from snapshot at the host where snapshot
# resides (boolean value)
#snapshot_same_host=true

# Ensure that the new volumes are the same AZ as snapshot or
# source volume (boolean value)
#cloned_volume_same_az=true


#
# Options defined in cinder.volume.driver
#

# The maximum number of times to rescan iSER targetto find
# volume (integer value)
#num_iser_scan_tries=3

# This option is deprecated and unused. It will be removed in
# the Liberty release. (integer value)
#iser_num_targets=<None>

# Prefix for iSER volumes (string value)
#iser_target_prefix=iqn.2010-10.org.openstack:

# The IP address that the iSER daemon is listening on (string
# value)
#iser_ip_address=$my_ip

# Prefix for iSCSI volumes (string value)
#iscsi_target_prefix=iqn.2010-10.org.openstack:

# The IP address that the iSCSI daemon is listening on (string
# value)
#iscsi_ip_address=$my_ip

# The list of secondary IP addresses of the iSCSI daemon (list
# value)
#iscsi_secondary_ip_addresses=

# The port that the iSCSI daemon is listening on (integer
# value)
#iscsi_port=3260

# The maximum number of times to rescan targets to find volume
# (integer value)
# Deprecated group/name - [DEFAULT]/num_iscsi_scan_tries
#num_volume_device_scan_tries=3

# The backend name for a given driver implementation (string
# value)
#volume_backend_name=<None>

# Do we attach/detach volumes in cinder using multipath for
# volume to image and image to volume transfers? (boolean
# value)
#use_multipath_for_image_xfer=false

# If this is set to True, attachment of volumes for image
# transfer will be aborted when multipathd is not running.
# Otherwise, it will fallback to single path. (boolean value)
#enforce_multipath_for_image_xfer=false

# Method used to wipe old volumes (string value)
#volume_clear=zero

# Size in MiB to wipe at start of old volumes. 0 => all
# (integer value)
#volume_clear_size=0

# The flag to pass to ionice to alter the i/o priority of the
# process used to zero a volume after deletion, for example
# "-c3" for idle only priority. (string value)
#volume_clear_ionice=<None>

# iSCSI target user-land tool to use. tgtadm is default, use
# lioadm for LIO iSCSI support, scstadmin for SCST target
# support, iseradm for the ISER protocol, ietadm for iSCSI
# Enterprise Target, iscsictl for Chelsio iSCSI Target or fake
# for testing. (string value)
#iscsi_helper=tgtadm

# Volume configuration file storage directory (string value)
#volumes_dir=$state_path/volumes

# IET configuration file (string value)
#iet_conf=/etc/iet/ietd.conf

# Chiscsi (CXT) global defaults configuration file (string
# value)
#chiscsi_conf=/etc/chelsio-iscsi/chiscsi.conf

# This option is deprecated and unused. It will be removed in
# the next release. (string value)
#lio_initiator_iqns=

# Sets the behavior of the iSCSI target to either perform
# blockio or fileio optionally, auto can be set and Cinder
# will autodetect type of backing device (string value)
#iscsi_iotype=fileio

# The default block size used when copying/clearing volumes
# (string value)
#volume_dd_blocksize=1M

# The blkio cgroup name to be used to limit bandwidth of
# volume copy (string value)
#volume_copy_blkio_cgroup_name=cinder-volume-copy

# The path to the client certificate key for verification, if
# the driver supports it. (string value)
#driver_client_cert_key=<None>

# Option to enable/disable CHAP authentication for targets.
# (boolean value)
# Deprecated group/name - [DEFAULT]/eqlx_use_chap
#use_chap_auth=false

# CHAP user name. (string value)
# Deprecated group/name - [DEFAULT]/eqlx_chap_login
#chap_username=

# Password for specified CHAP account name. (string value)
# Deprecated group/name - [DEFAULT]/eqlx_chap_password
#chap_password=

# Namespace for driver private data values to be saved in.
# (string value)
#driver_data_namespace=<None>

# String representation for an equation that will be used to
# filter hosts. Only used when the driver filter is set to be
# used by the Cinder scheduler. (string value)
#filter_function=<None>

# String representation for an equation that will be used to
# determine the goodness of a host. Only used when using the
# goodness weigher is set to be used by the Cinder scheduler.
# (string value)
#goodness_function=<None>


#
# Options defined in cinder.volume.drivers.block_device
#

# List of all available devices (list value)
#available_devices=


#
# Options defined in cinder.volume.drivers.cloudbyte.options
#

# These values will be used for CloudByte storage's addQos API
# call. (dict value)
#cb_add_qosgroup=latency:15,iops:10,graceallowed:false,iopscontrol:true,memlimit:0,throughput:0,tpcontrol:false,networkspeed:0

# Driver will use this API key to authenticate against the
# CloudByte storage's management interface. (string value)
#cb_apikey=None

# CloudByte storage specific account name. This maps to a
# project name in OpenStack. (string value)
#cb_account_name=None

# This corresponds to the name of Tenant Storage Machine (TSM)
# in CloudByte storage. A volume will be created in this TSM.
# (string value)
#cb_tsm_name=None

# A retry value in seconds. Will be used by the driver to
# check if volume creation was successful in CloudByte
# storage. (integer value)
#cb_confirm_volume_create_retry_interval=5

# Will confirm a successful volume creation in CloudByte
# storage by making this many number of attempts. (integer
# value)
#cb_confirm_volume_create_retries=3

# These values will be used for CloudByte storage's
# createVolume API call. (dict value)

# VNX authentication scope type. (string value)
#storage_vnx_authentication_type=global

# Directory path that contains the VNX security file. Make
# sure the security file is generated first. (string value)
#storage_vnx_security_file_dir=<None>

# Naviseccli Path. (string value)
#naviseccli_path=

# Storage pool name. (string value)
#storage_vnx_pool_name=<None>

# VNX secondary SP IP Address. (string value)
#san_secondary_ip=<None>

# Default timeout for CLI operations in minutes. For example,
# LUN migration is a typical long running operation, which
# depends on the LUN size and the load of the array. An upper
# bound in the specific deployment can be set to avoid
# unnecessary long wait. By default, it is 365 days long.
# (integer value)
#default_timeout=525600

# Default max number of LUNs in a storage group. By default,
# the value is 255. (integer value)
#max_luns_per_storage_group=255

# To destroy storage group when the last LUN is removed from
# it. By default, the value is False. (boolean value)
#destroy_empty_storage_group=false

# Mapping between hostname and its iSCSI initiator IP
# addresses. (string value)
#iscsi_initiators=

# Automatically register initiators. By default, the value is
# False. (boolean value)
#initiator_auto_registration=false

# Automatically deregister initiators after the related
# storage group is destroyed. By default, the value is False.
# (boolean value)
#initiator_auto_deregistration=false

# Report free_capacity_gb as 0 when the limit to maximum
# number of pool LUNs is reached. By default, the value is
# False. (boolean value)
#check_max_pool_luns_threshold=false

# Delete a LUN even if it is in Storage Groups. (boolean
# value)
#force_delete_lun_in_storagegroup=false
#
# Options defined in cinder.volume.drivers.emc.xtremio
#

# XMS cluster id in multi-cluster environment (string value)
#xtremio_cluster_name=


#
# Options defined in cinder.volume.drivers.eqlx
#

# Group name to use for creating volumes. Defaults to
# "group-0". (string value)
#eqlx_group_name=group-0

# Timeout for the Group Manager cli command execution. Default
# is 30. (integer value)
#eqlx_cli_timeout=30

# Maximum retry count for reconnection. Default is 5. (integer
# value)
#eqlx_cli_max_retries=5

# Use CHAP authentication for targets. Note that this option
# is deprecated in favour of "use_chap_auth" as specified in
# cinder/volume/driver.py and will be removed in next release.
# (boolean value)
eqlx_use_chap=false

# Existing CHAP account name. Note that this option is
# deprecated in favour of "chap_username" as specified in
# cinder/volume/driver.py and will be removed in next release.
# (string value)
#eqlx_chap_login=admin

# Password for specified CHAP account name. Note that this
# option is deprecated in favour of "chap_password" as
# specified in cinder/volume/driver.py and will be removed in
# the next release (string value)
#eqlx_chap_password=password

# Pool in which volumes will be created. Defaults to
# "default". (string value)
#eqlx_pool=default


#
# Options defined in cinder.volume.drivers.glusterfs
#

# File with the list of available gluster shares (string
# value)
#glusterfs_shares_config=/etc/cinder/glusterfs_shares

# Create volumes as sparsed files which take no space.If set
# to False volume is created as regular file.In such case
# volume creation takes a lot of time. (boolean value)
#glusterfs_sparsed_volumes=true

# Create volumes as QCOW2 files rather than raw files.
# (boolean value)
#glusterfs_qcow2_volumes=false

# Base dir containing mount points for gluster shares. (string
# value)
#glusterfs_mount_point_base=$state_path/mnt


#
# Options defined in cinder.volume.drivers.hds.hds
#

# The configuration file for the Cinder HDS driver for HUS
# (string value)
#hds_cinder_config_file=/opt/hds/hus/cinder_hus_conf.xml


#
# Options defined in cinder.volume.drivers.hds.iscsi
#

# Configuration file for HDS iSCSI cinder plugin (string
# value)
#hds_hnas_iscsi_config_file=/opt/hds/hnas/cinder_iscsi_conf.xml


#
# Options defined in cinder.volume.drivers.hds.nfs
#
# Interval to check copy (integer value)
#hitachi_copy_check_interval=3

# Interval to check copy asynchronously (integer value)
#hitachi_async_copy_check_interval=10

# Control port names for HostGroup or iSCSI Target (string
# value)
#hitachi_target_ports=<None>

# Range of group number (string value)
#hitachi_group_range=<None>

# Request for creating HostGroup or iSCSI Target (boolean
# value)
#hitachi_group_request=false


#
# Options defined in cinder.volume.drivers.hitachi.hbsd_fc
#

# Request for FC Zone creating HostGroup (boolean value)
#hitachi_zoning_request=false

# If False fully disable profiling feature. (boolean value)
#profiler_enabled=false

# If False doesn't trace SQL requests. (boolean value)
#trace_sqlalchemy=false

[lvm]
iscsi_helper=lioadm
volume_group=cinder-volumes
iscsi_ip_address=192.168.88.10
volume_driver=cinder.volume.drivers.lvm.LVMVolumeDriver
volumes_dir=/var/lib/cinder/volumes
iscsi_protocol=iscsi
volume_backend_name=lvm

"""

osp = OSP()
osp.role = "Controller"


def test_match():
    result = CinderConf(context_wrap(cinder_content, osp=osp))
    assert result.data.get("DEFAULT", "enabled_backends") == "tripleo_ceph"
    assert result.data.get("DEFAULT", "glance_api_ssl_compression") == "False"
    assert result.data.get("DEFAULT", "eqlx_use_chap") == "false"

    assert result.data.get("lvm", "iscsi_helper") == "lioadm"
    assert result.data.get("lvm", "volumes_dir") == "/var/lib/cinder/volumes"
    assert result.data.get("lvm", "volume_backend_name") == "lvm"



"""
Satellite MongoDB Commands
==========================

Parsers included in this module are:

MongoDBStorageEngine - command ``mongo pulp_database --eval 'db.serverStatus().storageEngine'``
-----------------------------------------------------------------------------------------------
MongoDBNonYumTypeRepos - command ``mongo pulp_database --eval 'db.repo_importers.find({"importer_type_id": { $ne: "yum_importer"}}).count()'``
----------------------------------------------------------------------------------------------------------------------------------------------

"""
from insights import parser, CommandParser
from insights.parsers import SkipException, ParseException
from insights.specs import Specs


@parser(Specs.satellite_mongodb_storage_engine)
class MongoDBStorageEngine(CommandParser, dict):
    """
    Read the ``mongo pulp_database --eval 'db.serverStatus().storageEngine'`` command
    and save the storage engine attributes to a dict.

    Sample Output::

        MongoDB shell version v3.4.9
        connecting to: mongodb://127.0.0.1:27017/pulp_database
        MongoDB server version: 3.4.9
        {
                "name" : "wiredTiger",
                "supportsCommittedReads" : true,
                "readOnly" : false,
                "persistent" : true
        }

    Examples::

        >>> type(satellite_storage_engine)
        <class 'insights.parsers.satellite_mongodb.MongoDBStorageEngine'>
        >>> satellite_storage_engine['name']
        'wiredTiger'

    Raises::

        SkipException: When there is no attribute in the output
        ParseException: When the storage engine attributes aren't in expected format
    """

    def parse_content(self, content):
        start_parse = False
        for line in content:
            line = line.strip()
            if not start_parse and line == '{':
                start_parse = True
                continue
            if start_parse and line == '}':
                break
            if start_parse:
                try:
                    name, value = [i.strip(' ,"') for i in line.split(':', 1)]
                    self[name] = value
                except Exception:
                    raise ParseException("Unable to parse the line: {0}".format(line))
        if not self:
            raise SkipException("Cannot get storage engine from Satellite MongoDB")


@parser(Specs.satellite_non_yum_type_repos)
class MongoDBNonYumTypeRepos(CommandParser):
    """
    Read the ``mongo pulp_database --eval 'db.repo_importers.find({"importer_type_id": { $ne: "yum_importer"}}).count()'`` command
    and save the count to attribute named count.

    Sample Output::

        MongoDB shell version v3.4.9
        connecting to: mongodb://127.0.0.1:27017/pulp_database
        MongoDB server version: 3.4.9
        0

    Examples::

        >>> type(satellite_non_yum_type_repos)
        <class 'insights.parsers.satellite_mongodb.MongoDBNonYumTypeRepos'>
        >>> satellite_non_yum_type_repos.count
        0

    Attributes::

        count (int): The count of non-yum type repos in pulp_database

    Raises::

        SkipException: When the output isn't in exptected format
    """

    def parse_content(self, content):
        if len(content) != 4 or not content[3].isdigit():
            raise SkipException("Unexpected output for MongoDBNonYumTypeRepos")
        self.count = int(content[3])



"""
Custom datasources for cloud initialization information
"""
import json
import yaml

from insights.core.context import HostContext
from insights.core.dr import SkipComponent
from insights.core.plugins import datasource
from insights.core.spec_factory import DatasourceProvider, simple_file
from insights.specs import Specs


class LocalSpecs(Specs):
    """ Local specs used only by cloud_init datasources """

    cloud_cfg_input = simple_file("/etc/cloud/cloud.cfg")
    """ Returns the contents of the file ``/etc/cloud/cloud.cfg`` """


@datasource(LocalSpecs.cloud_cfg_input, HostContext)
def cloud_cfg(broker):
    """
    This datasource provides the network configuration information collected
    from ``/etc/cloud/cloud.cfg``.

    Typical content of ``/etc/cloud/cloud.cfg`` file is::

        #cloud-config
        users:
          - name: demo
            ssh-authorized-keys:
              - key_one
              - key_two
            passwd: $6$j212wezy$7H/1LT4f9/N3wpgNunhsIqtMj62OKiS3nyNwuizouQc3u7MbYCarYeAHWYPYb2FT.lbioDm2RrkJPb9BZMN1O/

        network:
            version: 1
            config:
            - type: physical
                name: eth0
                subnets:
                  - type: dhcp
                  - type: dhcp6

        system_info:
            default_user:
            name: user2
            plain_text_passwd: 'someP@assword'
            home: /home/user2

        debug:
            output: /var/log/cloud-init-debug.log
            verbose: true

    Note:
        This datasource may be executed using the following command:

        ``insights cat --no-header cloud_cfg``

    Sample data returned includes only the ``network`` portion of the input file in JSON format::

        {
            "version": 1,
            "config": [
                {
                    "type": "physical",
                    "name": "eth0",
                    "subnets": [
                        {"type": "dhcp"},
                        {"type": "dhcp6"}
                    ]
                }
            ]
        }

    Returns:
        str: JSON string when the ``network`` parameter includes content, else `None` is returned.

    Raises:
        SkipComponent: When the path does not exist or any exception occurs.
    """
    relative_path = '/etc/cloud/cloud.cfg'
    try:
        content = broker[LocalSpecs.cloud_cfg_input].content
        if content:
            content = yaml.load('\n'.join(content), Loader=yaml.SafeLoader)
            network_config = content.get('network', None)
            if network_config:
                return DatasourceProvider(content=json.dumps(network_config), relative_path=relative_path)
    except Exception as e:
        raise SkipComponent("Unexpected exception:{e}".format(e=str(e)))

    raise SkipComponent('No network section in yaml')



import string
from insights.parsr import (Char, EOF, EOL, EndTagName, Forward, FS, GT, LT,
        Letters, Lift, LineEnd, Many, Number, OneLineComment, PosMarker,
        QuotedString, skip_none, StartTagName, String, WS, WSChar)
from insights.parsr.query import Directive, Entry, Section


def loads(data):
    return Entry(children=Top(data)[0])


def load(f):
    return loads(f.read())


def simple_to_entry(name, attrs):
    return Directive(name=name.value, attrs=attrs, lineno=name.lineno)


def complex_to_entry(tag, children):
    name, attrs = tag
    return Section(name=name.value, attrs=attrs, children=children, lineno=name.lineno)


Complex = Forward()
Num = Number & (WSChar | LineEnd)
Cont = Char("\\") + EOL
StartName = WS >> PosMarker(StartTagName(Letters)) << WS
EndName = WS >> EndTagName(Letters) << WS
Comment = (WS >> OneLineComment("#")).map(lambda x: None)
AttrStart = Many(WSChar)
AttrEnd = (Many(WSChar) + Cont) | Many(WSChar)
BareAttr = String(set(string.printable) - (set(string.whitespace) | set(";{}<>\\'\"")))
Attr = AttrStart >> (Num | BareAttr | QuotedString) << AttrEnd
Attrs = Many(Attr)
StartTag = (WS + LT) >> (StartName + Attrs) << (GT + WS)
EndTag = (WS + LT + FS) >> EndName << (GT + WS)
Simple = WS >> (Lift(simple_to_entry) * PosMarker(Letters) * Attrs) << WS
Stanza = Simple | Complex | Comment
Complex <= (Lift(complex_to_entry) * StartTag * Many(Stanza).map(skip_none)) << EndTag
Doc = Many(Stanza).map(skip_none)
Top = Doc + EOF



from insights.tests import context_wrap
from insights.parsers.sysconfig import NtpdSysconfig

NTPD = """
OPTIONS="-x -g"
#HIDE="me"
""".strip()


def test_sysconfig_ntpd():
    result = NtpdSysconfig(context_wrap(NTPD))
    assert result['OPTIONS'] == '-x -g'
    assert result.get('HIDE') is None



from insights.tests import context_wrap
from insights.parsers.multicast_querier import MulticastQuerier

MULTICAST_QUERIER = """
/sys/devices/virtual/net/br0/bridge/multicast_querier
0
/sys/devices/virtual/net/br1/bridge/multicast_querier
1
/sys/devices/virtual/net/br2/bridge/multicast_querier
0
"""


def test_mcast_queri():
    result = MulticastQuerier(context_wrap(MULTICAST_QUERIER))
    assert result.bri_val == {'br0': 0, 'br1': 1, 'br2': 0}
    assert result.bri_val['br1'] == 1
    assert len(result.bri_val) == 3



"""
audit_log - File ``/var/log/audit/audit.log``
=============================================
"""

import shlex
from datetime import date
from .. import LogFileOutput, parser, add_filter
from insights.specs import Specs

# Currently, only selinux related(AVC type) audit logs are interested.
# Add this filter in parser directly to filter out too many other types logs.
# Also, filters can be changed to meet any further requirments.
filter_list = [
    'type=AVC',
]
add_filter(Specs.audit_log, filter_list)


@parser(Specs.audit_log)
class AuditLog(LogFileOutput):
    """
    Class for parsing ``/var/log/audit/audit.log`` file.

    Sample log lines::

        type=CRYPTO_KEY_USER msg=audit(1506046832.641:53584): pid=16865 uid=0 auid=0 ses=7247 subj=unconfined_u:unconfined_r:unconfined_t:s0-s0:c0.c1023 msg='op=destroy kind=session fp=? direction=both spid=16865 suid=0 rport=59296 laddr=192.0.2.1 lport=22  exe="/usr/sbin/sshd" hostname=? addr=192.0.2.3 terminal=? res=success'
        type=LOGIN msg=audit(1506047401.407:53591): pid=482 uid=0 subj=system_u:system_r:crond_t:s0-s0:c0.c1023 old-auid=4294967295 auid=993 old-ses=4294967295 ses=7389 res=1
        type=AVC msg=audit(1506487181.009:32794): avc:  denied  { create } for  pid=27960 comm="mongod" scontext=system_u:system_r:mongod_t:s0 tcontext=system_u:system_r:mongod_t:s0 tclass=unix_dgram_socket

    Examples:
        >>> log = shared[AuditLog]
        >>> log.get('type=AVC')
        [{
            'is_valid': True,
            'timestamp': '1506487181.009',
            'unparsed': 'avc:  denied  { create } for',
            'msg_ID': '32794',
            'pid': '27960',
            'raw_message': 'type=AVC msg=audit(1506487181.009:32794): avc:  denied  { create } for  pid=27960 comm="mongod" scontext=system_u:system_r:mongod_t:s0 tcontext=system_u:system_r:mongod_t:s0 tclass=unix_dgram_socket',
            'comm': 'mongod',
            'scontext': 'system_u:system_r:mongod_t:s0',
            'tclass': 'unix_dgram_socket',
            'type': 'AVC',
            'tcontext': 'system_u:system_r:mongod_t:s0'
        }]
        >>> assert len(list(log.get_after(timestamp=date.fromtimestamp(1506047401.407)))) == 3
    """
    def _parse_line(self, line):
        """
        Parse a log line into a info dictionary.

        Parsing logic:

            * First, use `shlex.split` to split by empty.
            * Next, assert the first two key-value pair is 'type' and 'msg'.
            * Next, parse the remained string reversly to get key-value pair data as more as possible.
            * The left unparsed string will be stored at "unparsed".
            * At last, wrap all key-value pairs into a dict called info.
            * Note, if a line isn't parseable by the above steps, return dict with items 'raw_message' and 'is_valid' only.

        Parameters:
            line(string): Log line to parse

        Yields:
            Dictionary corresponding to the data in line split into
            functional elements:

            * **raw_message** - the original unparsed line
            * **is_valid** - true if the line is parseable, else false
            * **type** - the log type
            * **timestamp** - the timestamp of the log line (as a string)
            * **msg_ID** - the serial number behind the timestamp
            * **unparsed** - the unparsed part in log, may or maynot exist

            If the line is not regular, for some reason, then as many fields as
            possible are pulled from the line.
        """
        info = {'raw_message': line, 'is_valid': False}
        linesp = shlex.split(line)

        if (len(linesp) < 2 or
                not (linesp[0] and linesp[0].startswith('type=')) or
                not (linesp[1] and linesp[1].startswith('msg=audit('))):
            return info

        timestamp_id = linesp[1].lstrip('msg=audit(').rstrip('):')
        timestamp_id_sp = timestamp_id.split(':')
        if len(timestamp_id_sp) != 2:
            return info

        info['type'] = linesp[0].lstrip('type=')
        info['timestamp'] = timestamp_id_sp[0]
        info['msg_ID'] = timestamp_id_sp[1]

        for index in range(len(linesp) - 1, 1, -1):
            itemsp = linesp[index].split('=', 1)
            if len(itemsp) < 2:
                unparsed1 = (line.rsplit(linesp[index + 1])[0]
                             if index + 1 < len(linesp)
                             else line)
                unparsed2 = unparsed1.split(linesp[1])[-1]
                info["unparsed"] = unparsed2.strip()
                break
            info[itemsp[0]] = itemsp[1]

        info['is_valid'] = True
        return info

    def get_after(self, timestamp, s=None):
        """
        Find all the (available) logs that are after the given time stamp.
        Override this function in class LogFileOutput.

        Parameters:

            timestamp(datetime.datetime): lines before this time are ignored.
            s(str or list): one or more strings to search for.
                If not supplied, all available lines are searched.

        Yields:
            (dict): the parsed data of lines with timestamps after this date in the
            same format they were supplied.
        """
        search_by_expression = self._valid_search(s)
        for line in self.lines:
            # If `s` is not None, keywords must be found in the line
            if s and not search_by_expression(line):
                continue
            info = self._parse_line(line)
            try:
                logtime = date.fromtimestamp(float(info.get('timestamp', 0)))
                if logtime > timestamp:
                    yield info
            except:
                pass



"""
Apache httpd logs
=================

Modules for parsing the log files of httpd service.  Parsers include:

HttpdSSLErrorLog - file ``ssl_error_log``
-----------------------------------------

HttpdErrorLog - file ``error_log``
----------------------------------

Httpd24HTTPDErrorLog - file ``httpd24_httpd_error_log``
--------------------------------------------------------

JBCSHTTPD24HttpdErrorLog - file ``jbcs_httpd24_httpd_error_log``
-----------------------------------------------------------------

HttpdSSLAccessLog - file ``ssl_access_log``
-------------------------------------------

HttpdAccessLog - file ``access_log``
------------------------------------

.. note::
    Please refer to the super-class :py:class:`insights.core.LogFileOutput`
    for more usage information.

"""

from .. import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.httpd_ssl_error_log)
class HttpdSSLErrorLog(LogFileOutput):
    """Class for parsing httpd ``ssl_error_log`` file."""
    time_format = '%b %d %H:%M:%S.%f %Y'


@parser(Specs.httpd_error_log)
class HttpdErrorLog(LogFileOutput):
    """Class for parsing httpd ``error_log`` file."""
    time_format = '%b %d %H:%M:%S.%f %Y'


@parser(Specs.httpd24_httpd_error_log)
class Httpd24HttpdErrorLog(LogFileOutput):
    """Class for parsing httpd ``error_log`` file."""
    time_format = '%b %d %H:%M:%S.%f %Y'


@parser(Specs.jbcs_httpd24_httpd_error_log)
class JBCSHttpd24HttpdErrorLog(LogFileOutput):
    """Class for parsing httpd ``error_log`` file."""
    time_format = '%b %d %H:%M:%S.%f %Y'


@parser(Specs.httpd_ssl_access_log)
class HttpdSSLAccessLog(LogFileOutput):
    """Class for parsing httpd ``ssl_access_log`` file."""
    time_format = '%d/%b/%Y:%H:%M:%S'


@parser(Specs.httpd_access_log)
class HttpdAccessLog(LogFileOutput):
    """Class for parsing httpd ``access_log`` file."""
    time_format = '%d/%b/%Y:%H:%M:%S'



import pytest
from insights.parsers import ParseException
from insights.parsers.interrupts import Interrupts
from insights.tests import context_wrap

INT_MULTI = """
           CPU0       CPU1       CPU2       CPU3       CPU4       CPU5       CPU6       CPU7
  0:         37          0          0          0          0          0          0          0  IR-IO-APIC   2-edge      timer
  1:          3          2          1          0          3          3          0          3  IR-IO-APIC   1-edge      i8042
  8:          0          1          0          0          0          0          0          0  IR-IO-APIC   8-edge      rtc0
  9:      11107       2316       4040       1356       1324       1006        548        537  IR-IO-APIC   9-fasteoi   acpi
 12:         44         13          8         10          7         16        320         93  IR-IO-APIC  12-edge      i8042
 16:          4          2          2          8          0          6          7          0  IR-IO-APIC  16-fasteoi   ehci_hcd:usb1, mmc0
 18:          0          0          0          0          0          0          0          0  IR-IO-APIC  18-fasteoi   i801_smbus
 23:          6          6          4          4          0         10          3          0  IR-IO-APIC  23-fasteoi   ehci_hcd:usb2
 24:          0          0          0          0          0          0          0          0  DMAR-MSI   0-edge      dmar0
 25:          0          0          0          0          0          0          0          0  DMAR-MSI   1-edge      dmar1
 27:     242965     155333      52536      47894      35496      32961      27934      96634  IR-PCI-MSI 512000-edge      ahci[0000:00:1f.2]
 28:    2524672     433481     227987     212406     162014     175712     150609     151638  IR-PCI-MSI 327680-edge      xhci_hcd
 29:     974462     273360      84814     115282      66423      72322      48783      56791  IR-PCI-MSI 409600-edge      enp0s25
 30:        100         22         12         19         16          6         10          5  IR-PCI-MSI 524288-edge
 31:     321958      46760      42215      27668      34390      22665      29484      20514  IR-PCI-MSI 32768-edge      i915
 32:          7          1          2          7          0          3          0          6  IR-PCI-MSI 360448-edge      mei_me
 33:     193890      10271      22883       6823       3023       3918       2610       3798  IR-PCI-MSI 1572864-edge      iwlwifi
 34:        123         95         81         18         61          9         23         38  IR-PCI-MSI 442368-edge      snd_hda_intel:card2
 35:        460        102        375        269        123         48         38         23  IR-PCI-MSI 49152-edge      snd_hda_intel:card1
NMI:        210         92        179         96        177          0          0          0   Non-maskable interrupts
LOC:    7561411    2488524    6527767    2448192    6244091    2942086    6199519    2508884   Local timer interrupts
SPU:          0          0          0          0          0          0          0          0   Spurious interrupts
PMI:        210         92        179         96        177          0          0          0   Performance monitoring interrupts
IWI:          2          0          0          0          0          1          0          0   IRQ work interrupts
RTR:          0          0          0          0          0          0          0          0   APIC ICR read retries
RES:     181183     110341     106958      66026      92237      61031      72582      55959   Rescheduling interrupts
CAL:       2946       2842       2954       2995       2098       2835       2895       3016   Function call interrupts
TLB:     362051     255750     355120     266627     354757     264554     360694     266051   TLB shootdowns
TRM:          0          0          0          0          0          0          0          0   Thermal event interrupts
THR:          0          0          0          0          0          0          0          0   Threshold APIC interrupts
DFR:          0          0          0          0          0          0          0          0   Deferred Error APIC interrupts
MCE:          0          0          0          0          0          0          0          0   Machine check exceptions
MCP:        381        381        381        381        381        381        381        381   Machine check polls
ERR:          0
MIS:          0
PIN:          0          0          0          0          0          0          0          0   Posted-interrupt notification event
PIW:          0          0          0          0          0          0          0          0   Posted-interrupt wakeup event
""".strip()

INT_SINGLE = """
           CPU0
  0:         45   IO-APIC-edge      timer
  1:          3   IO-APIC-edge      i8042
  6:          3   IO-APIC-edge      floppy
  7:          0   IO-APIC-edge      parport0
  8:          1   IO-APIC-edge      rtc0
  9:          0   IO-APIC-fasteoi   acpi
HYP:          0   Hypervisor callback interrupts
ERR:          0
MIS:          0
""".strip()

INT_INVALID_1 = """
  0:         45   IO-APIC-edge      timer
  1:          3   IO-APIC-edge      i8042
  6:          3   IO-APIC-edge      floppy
""".strip()

INT_INVALID_2 = """


""".strip()

INT_NO_INTERRUPTS = """
           CPU0
"""


def test_interrupts():
    all_ints = Interrupts(context_wrap(INT_MULTI))
    assert len(all_ints.data) == 37
    assert all_ints.num_cpus == 8
    assert all_ints.get("i8042") == [all_ints.data[1], all_ints.data[4]]
    assert len(all_ints.get("IR-PCI-MSI")) == 9
    for one_int in all_ints:
        if one_int['irq'] == "NMI:":
            assert one_int['counts'] == ["210", "92", "179", "96", "177", "0", "0", "0"]
            assert one_int['type_device'] == "Non-maskable interrupts"
    all_ints = Interrupts(context_wrap(INT_SINGLE))
    assert len(all_ints.data) == 9
    for one_int in all_ints:
        assert len(one_int['counts']) == 1
    assert 'type_device' in all_ints.data[6]
    assert 'type_device' not in all_ints.data[7]
    assert 'type_device' not in all_ints.data[8]

    with pytest.raises(ParseException):
        Interrupts(context_wrap(INT_INVALID_1))

    with pytest.raises(ParseException):
        Interrupts(context_wrap(INT_INVALID_2))

    with pytest.raises(ParseException):
        Interrupts(context_wrap(INT_NO_INTERRUPTS))



import pytest
from mock.mock import Mock

from insights.core.dr import SkipComponent
from insights.core.spec_factory import DatasourceProvider
from insights.specs.datasources.ps import ps_eo_cmd, LocalSpecs

PS_DATA = """
PID COMMAND
  1 /usr/lib/systemd/systemd --switched-root --system --deserialize 31
  2 [kthreadd]
  3 [rcu_gp]
  4 [rcu_par_gp]
  6 [kworker/0:0H-events_highpri]
  9 [mm_percpu_wq]
 10 [rcu_tasks_kthre]
 11 /usr/bin/python3 /home/user1/python_app.py
 12 [kworker/u16:0-kcryptd/253:0]
"""

PS_EXPECTED = """
PID COMMAND
1 /usr/lib/systemd/systemd
2 [kthreadd]
3 [rcu_gp]
4 [rcu_par_gp]
6 [kworker/0:0H-events_highpri]
9 [mm_percpu_wq]
10 [rcu_tasks_kthre]
11 /usr/bin/python3
12 [kworker/u16:0-kcryptd/253:0]
"""

PS_BAD = "Command not found"

PS_EMPTY = """
PID COMMAND
"""

RELATIVE_PATH = 'insights_commands/ps_eo_cmd'


def test_ps_eo_cmd():
    ps_eo_args = Mock()
    ps_eo_args.content = PS_DATA.splitlines()
    broker = {LocalSpecs.ps_eo_args: ps_eo_args}
    result = ps_eo_cmd(broker)
    assert result is not None
    assert isinstance(result, DatasourceProvider)
    expected = DatasourceProvider(content=PS_EXPECTED.strip(), relative_path=RELATIVE_PATH)
    assert result.content == expected.content
    assert result.relative_path == expected.relative_path


def test_ps_eo_cmd_bad():
    ps_eo_args = Mock()
    ps_eo_args.content = PS_BAD.splitlines()
    broker = {LocalSpecs.ps_eo_args: ps_eo_args}
    with pytest.raises(SkipComponent) as e:
        ps_eo_cmd(broker)
    assert e is not None


def test_ps_eo_cmd_empty():
    ps_eo_args = Mock()
    ps_eo_args.content = PS_EMPTY.splitlines()
    broker = {LocalSpecs.ps_eo_args: ps_eo_args}
    with pytest.raises(SkipComponent) as e:
        ps_eo_cmd(broker)
    assert e is not None



"""
Nmcli parsers
=============

This file will parse the output of command line tools used to manage
NetworkManager.

Parsers provided by this module include:

NmcliDevShow - command ``/usr/bin/nmcli dev show``
--------------------------------------------------
NmcliConnShow - command ``/usr/bin/nmcli conn show``
----------------------------------------------------
"""


import re
from insights import parser, get_active_lines, CommandParser
from insights.specs import Specs
from insights.parsers import parse_delimited_table, SkipException


@parser(Specs.nmcli_dev_show)
class NmcliDevShow(CommandParser, dict):
    """
    .. warning::
        This parser may be for a single device, please use
        :py:class:`insights.combiners.nmcli.AllNmcliDevShow` instead for all the
        devices.

    This class will parse the output of command ``nmcli dev show``, and the information
    will be stored in dictionary format.

    NetworkManager displays all the devices and their current states along with network
    configuration and connection status.

    This parser works like a python dictionary, all parsed data can be accessed
    via the ``dict`` interfaces.

    Sample input for ``/usr/bin/nmcli dev show``::

        GENERAL.DEVICE:                         em3
        GENERAL.TYPE:                           ethernet
        GENERAL.HWADDR:                         B8:AA:BB:DE:F8:B9
        GENERAL.MTU:                            1500
        GENERAL.STATE:                          100 (connected)
        GENERAL.CONNECTION:                     em3
        GENERAL.CON-PATH:                       /org/freedesktop/NetworkManager/ActiveConnection/1
        WIRED-PROPERTIES.CARRIER:               on
        IP4.ADDRESS[1]:                         10.16.184.98/22
        IP4.GATEWAY:                            10.16.187.254
        IP4.DNS[1]:                             10.16.36.29
        IP4.DNS[2]:                             10.11.5.19
        IP4.DNS[3]:                             10.5.30.160
        IP4.DOMAIN[1]:                          abc.lab.eng.example.com
        IP6.ADDRESS[1]:                         2620:52:0:10bb:ba2a:72ff:fede:f8b9/64
        IP6.ADDRESS[2]:                         fe80::ba2a:72ff:fede:f8b9/64
        IP6.GATEWAY:                            fe80:52:0:10bb::fc
        IP6.ROUTE[1]:                           dst = 2620:52:0:10bb::/64, nh = ::, mt = 100

        GENERAL.DEVICE:                         em1
        GENERAL.TYPE:                           ethernet
        GENERAL.HWADDR:                         B8:AA:BB:DE:F8:BB
        GENERAL.MTU:                            1500
        GENERAL.STATE:                          30 (disconnected)
        GENERAL.CONNECTION:                     --
        GENERAL.CON-PATH:                       --
        WIRED-PROPERTIES.CARRIER:               off

        GENERAL.DEVICE:                         em2
        GENERAL.TYPE:                           ethernet
        GENERAL.HWADDR:                         B8:AA:BB:DE:F8:BC
        GENERAL.MTU:                            1500
        GENERAL.STATE:                          30 (disconnected)
        GENERAL.CONNECTION:                     --
        GENERAL.CON-PATH:                       --
        WIRED-PROPERTIES.CARRIER:               off

    Examples:
        >>> type(nmcli_obj)
        <class 'insights.parsers.nmcli.NmcliDevShow'>
        >>> nmcli_obj['em3']['STATE']
        'connected'
        >>> nmcli_obj.get('em2')['HWADDR']
        'B8:AA:BB:DE:F8:BC'
        >>> sorted(nmcli_obj.connected_devices)
        ['em1', 'em2', 'em3']

    """
    def parse_content(self, content):
        if not content:
            raise SkipException()

        data = {}
        per_device = {}
        current_dev = ""
        for line in get_active_lines(content):
            if (not ("not found" in line or "Error" in line or "No such file" in line or "Warning" in line)):
                if len(line.split(": ")) >= 2:
                    key, val = line.split(": ")
                    if "IP" in key:
                        proto = re.sub(r'\[|\]', r'', key.split('.')[1])
                        key = key.split('.')[0] + "_" + proto
                    else:
                        key = key.split('.')[1]
                    val = re.sub(r'\d+\s|\(|\)', r'', val.strip())

                    # Device configuration details starts here
                    if key == "DEVICE" and not current_dev:
                        current_dev = val
                        continue
                    elif key == "DEVICE" and current_dev:
                        data[current_dev] = per_device
                        current_dev = val
                        per_device = {}
                        continue
                    per_device.update({key: val})
        if current_dev and per_device:
            # Last device configuration details
            data[current_dev] = per_device

        if not data:
            raise SkipException()
        self.update(data)
        self._con_dev = [k for k, v in data.items()
                         if 'STATE' in v and v['STATE'] == 'connected']

    @property
    def connected_devices(self):
        """(list): The list of devices who's state is connected and managed by NetworkManager"""
        return self._con_dev

    @property
    def data(self):
        """(dict): Dict with the device name as the key and device details as the value."""
        return self


@parser(Specs.nmcli_dev_show_sos)
class NmcliDevShowSos(NmcliDevShow):
    """
    .. warning::
        This parser may be for a single device, please use
        :py:class:`insights.combiners.nmcli.AllNmcliDevShow` instead for all the
        devices.

    In some versions of sosreport, the ``nmcli dev show`` command is separated
    to individual files for different devices. While in some versions, it's
    still a whole file.  The base class :py:class:`NmcliDevShow` could handle
    both of them, except that the parsing results is stored into a list for
    separated files.

    """
    pass


@parser(Specs.nmcli_conn_show)
class NmcliConnShow(CommandParser):
    """
    This file will parse the output of all the nmcli connections.

    Sample configuration from a teamed interface in file ``/usr/bin/nmcli conn show``::

       NAME      UUID                                  TYPE      DEVICE
       enp0s3    320d4923-c410-4b22-b7e9-afc5f794eecc  ethernet  enp0s3
       virbr0    7c7dec66-4a8c-4b49-834a-889194b3b83c  bridge    virbr0
       test-net  f858b1cc-d149-4de0-93bc-b1826256847a  ethernet  --

    Examples:
        >>> type(static_conn)
        <class 'insights.parsers.nmcli.NmcliConnShow'>
        >>> static_conn.disconnected_connection
        ['test-net-1']

    Attributes:
        data (list): list of connections wrapped in dictionaries

    """
    def parse_content(self, content):
        try:
            self.data = parse_delimited_table(content, heading_ignore=["NAME", "UUID", "TYPE", "DEVICE"])
        except:
            raise SkipException("Invalid Contents!")
        self._disconnected_connection = []
        for all_connection in self.data:
            if all_connection['DEVICE'] == "--":
                self._disconnected_connection.append(all_connection['NAME'])

    @property
    def disconnected_connection(self):
        """(list): It will return all the disconnected static route connections."""
        return self._disconnected_connection



"""
NeutronDhcpAgentIni - file ``/etc/neutron/dhcp_agent.ini``
==========================================================

The ``NeutronDhcpAgentIni`` class parses the dhcp-agent configuration file.
See the ``IniConfigFile`` class for more usage information.
"""

from insights import parser, IniConfigFile
from insights.core.filters import add_filter
from insights.specs import Specs

add_filter(Specs.neutron_dhcp_agent_ini, ["["])


@parser(Specs.neutron_dhcp_agent_ini)
class NeutronDhcpAgentIni(IniConfigFile):
    """
    Parse the ``/etc/neutron/dhcp_agent.ini`` configuration file.

    Sample configuration::

        [DEFAULT]

        ovs_integration_bridge = br-int
        ovs_use_veth = false
        interface_driver = neutron.agent.linux.interface.OVSInterfaceDriver
        ovs_vsctl_timeout = 10
        resync_interval = 30
        dhcp_driver = neutron.agent.linux.dhcp.Dnsmasq
        enable_isolated_metadata = True
        force_metadata = False
        enable_metadata_network = False
        root_helper=sudo neutron-rootwrap /etc/neutron/rootwrap.conf
        state_path=/var/lib/neutron

        [AGENT]

        report_interval = 30
        log_agent_heartbeats = false
        availability_zone = nova


    Examples:

        >>> data.has_option("AGENT", "log_agent_heartbeats")
        True
        >>> data.get("DEFAULT", "force_metadata") == "True"
        True
        >>> data.getint("DEFAULT", "resync_interval")
        30

    """
    pass



from __future__ import absolute_import
import six
import logging
import textwrap

from .constants import InsightsConstants as constants

APP_NAME = constants.app_name
logger = logging.getLogger(__name__)


def map_rm_conf_to_components(rm_conf, uploader_json):
    '''
    In order to maximize compatibility between "classic" remove.conf
    configurations and core collection, do the following mapping
    strategy:
    1. If remove.conf entry matches a symbolic name, disable the
        corresponding core component.
    2. If remove.conf entry is a raw command or file, do a reverse
        lookup on the symbolic name based on stored uploader.json data,
        then continue as in step 1.
    3. If neither conditions 1 or 2 are matched it is either
        a) a mistyped command/file, or
        b) an arbitrary file.
        For (a), classic remove.conf configs require an exact match to
            uploader.json. We can carry that condition into our
            compatibility with core.
        For (b), classic collection had the ability to skip arbitrary
            files based on filepaths in uploader.json post-expansion
            (i.e. a specific repo file in /etc/yum.repos.d).
            Core checks all files collected against the file
            blacklist filters, so these files will be omitted
            just by the nature of core collection.
    '''
    updated_commands = []
    updated_files = []
    updated_components = []

    if not rm_conf:
        return rm_conf

    logger.warning("If possible, commands and files specified in the blacklist configuration will be converted to Insights component specs that will be disabled as needed.")

    # save matches to a dict for informative logging
    conversion_map = {}
    longest_key_len = 0

    for section in ['commands', 'files']:
        if section not in rm_conf:
            continue
        for key in rm_conf[section]:
            if section == 'commands':
                symbolic_name = _search_uploader_json(uploader_json, ['commands'], key)
            elif section == 'files':
                # match both files and globs to rm_conf files
                symbolic_name = _search_uploader_json(uploader_json, ['files', 'globs'], key)

            component = _get_component_by_symbolic_name(symbolic_name)
            if component:
                conversion_map[key] = component
                if len(key) > longest_key_len:
                    longest_key_len = len(key)
                updated_components.append(component)
            else:
                if section == 'commands':
                    updated_commands.append(key)
                elif section == 'files':
                    updated_files.append(key)

    _log_conversion_table(conversion_map, longest_key_len)

    if 'components' in rm_conf:
        # update components list if there already is one
        original_comp_set = set(rm_conf['components'])
        updated_comp_set = set(dict.fromkeys(updated_components))
        # avoid duplicates
        rm_conf['components'] += list(updated_comp_set - original_comp_set)
    else:
        # otherwise create it
        rm_conf['components'] = list(dict.fromkeys(updated_components))

    rm_conf['commands'] = updated_commands
    rm_conf['files'] = updated_files

    return rm_conf


def _search_uploader_json(uploader_json, headings, key):
    '''
    Search an uploader.json block for a command/file from "name"
    and return the symbolic name if it exists

    headings        - list of headings to search inside uploader.json
    key             - raw command/file or symbolic name to search
    conversion_map  - list of names to found components for logging
    longest_key_len - length of longest name for logging
    '''
    for heading in headings:
        # keys inside the dicts are the heading, but singular
        singular = heading.rstrip('s')

        for spec in uploader_json[heading]:
            if key == spec['symbolic_name'] or (key == spec[singular] and heading != 'globs'):
                # matches to a symbolic name or raw command, cache the symbolic name
                # only match symbolic name for globs
                sname = spec['symbolic_name']
                if not six.PY3:
                    sname = sname.encode('utf-8')
                return sname
    # no match
    return None


def _get_component_by_symbolic_name(sname):
    # match a component to a symbolic name
    # some symbolic names need to be renamed to fit specs
    if sname is None:
        # toss back bad input
        return None

    spec_prefix = "insights.specs.default.DefaultSpecs."
    spec_conversion = {
        'getconf_pagesize': 'getconf_page_size',
        'netstat__agn': 'netstat_agn',
        'rpm__V_packages': 'rpm_V_packages',

        'machine_id1': 'machine_id',
        'machine_id2': 'machine_id',
        'machine_id3': 'machine_id',
        'grub2_efi_grubenv': None,
        'grub2_grubenv': None,
        'limits_d': 'limits_conf',
        'modprobe_conf': 'modprobe',
        'modprobe_d': 'modprobe',
        'ps_auxwww': 'insights.specs.sos_archive.SosSpecs.ps_auxww',  # special case
        'rh_mongodb26_conf': 'mongod_conf',
        'sysconfig_rh_mongodb26': 'sysconfig_mongod',
        'redhat_access_proactive_log': None,

        'krb5_conf_d': 'krb5'
    }

    if sname in spec_conversion:
        if spec_conversion[sname] is None:
            return None
        if sname == 'ps_auxwww':
            return spec_conversion[sname]
        return spec_prefix + spec_conversion[sname]
    return spec_prefix + sname


def _log_conversion_table(conversion_map, longest_key_len):
    '''
    Handle wrapping & logging the conversions
    '''
    max_log_len = 48

    for n in conversion_map:
        spec_name_no_prefix = conversion_map[n].rsplit('.', 1)[-1]

        # for specs exceeding a max length, wrap them past the first line
        if longest_key_len > max_log_len:
            log_len = max_log_len
        else:
            log_len = longest_key_len

        wrapped_spec = textwrap.wrap(n, max_log_len)
        # log the conversion on the first line of the "wrap"
        wrapped_spec[0] = '- {0:{1}} => {2}'.format(wrapped_spec[0], log_len, spec_name_no_prefix)
        logger.warning('\n  '.join(wrapped_spec))



"""
Get ``uid`` of user ``nova`` and ``nova_migration``
===================================================

The parser class in this module uses base parser class
``CommandParser`` to get uids of the user ``nova`` and
``nova_migration``.

Parsers included in this module are:

NovaUID - command ``id -u nova``
--------------------------------

NovaMigrationUID - command ``id -u nova_migration``
---------------------------------------------------
"""
from insights import CommandParser, parser
from insights.parsers import ParseException, SkipException
from insights.specs import Specs


@parser(Specs.nova_uid)
class NovaUID(CommandParser):
    '''Parse output of ``id -u nova`` and get the ``uid`` (int).

    Typical output of the ``id -u nova`` command is::

        162

    However the id number may vary.

    Examples:

        >>> nova_uid.data
        162

    Attributes:
        data: ``int`` if 'nova' user exist.

    Raises:
        SkipException: If 'nova' user not found or output is empty.
        ParseException: For any other output which is not a number or multi-line.
            Outputs of such kind are not yet expected from the command `id`.
    '''
    def parse_content(self, content):
        if not content:
            raise SkipException("Empty output.")
        if "no such user" in content[0].lower():
            raise SkipException("No such user.")
        if len(content) > 1 or not content[0].isdigit():
            raise ParseException("Unable to parse user ID: {0}".format(content))
        self.data = int(content[0])


@parser(Specs.nova_migration_uid)
class NovaMigrationUID(NovaUID):
    '''Parse output of ``id -u nova_migration`` and get the ``uid`` (int).

    Typical output of the ``id -u nova_migration`` command is::

        153

    However the id number may vary.

    Examples:

        >>> nova_migration_uid.data
        153

    Attributes:
        data: ``int`` if 'nova_migration' user exist.

    Raises:
        SkipException: If 'nova_migration' user not found or output is empty.
        ParseException: For any other output which is not a number or multi-line.
            Output of such kind are not yet expected from the command `id`.
    '''
    pass



"""
CupsPpd - files ``/etc/cups/ppd/*``
===================================

Parser to parse the content of files ``/etc/cups/ppd/*``
"""

from insights import Parser
from insights import parser
from insights.specs import Specs
from insights.parsers import SkipException


@parser(Specs.cups_ppd)
class CupsPpd(Parser, dict):
    """
    Class to parse ``/etc/cups/ppd/*`` files.

    Sample output for files::

        *PPD-Adobe: "4.3"
        *% Copyright 2007-2014 by Apple Inc.
        *% Copyright: 1997-2007 by Easy Software Products.
        *FormatVersion: "4.3"
        *FileVersion: "2.2"
        *LanguageVersion: English
        *LanguageEncoding: ISOLatin1
        *PCFileName: "ippeve.ppd"
        *Manufacturer: "Canon"
        *ModelName: "iR-ADV C3525/3530 PPD"
        *Product: "(iR-ADV C3525/3530 PPD)"
        *NickName: "iR-ADV C3525/3530 PPD"
        *ShortNickName: "iR-ADV C3525/3530 PPD"
        *cupsFilter2: "application/vnd.cups-pdf application/pdf 10 -"
        *cupsFilter2: "application/vnd.cups-postscript application/postscript 10 -"

    Examples:
        >>> type(cups_ppd)
        <class 'insights.parsers.cups_ppd.CupsPpd'>
        >>> cups_ppd["PCFileName"]
        '"ippeve.ppd"'
        >>> cups_ppd["cupsFilter2"]
        ['"application/vnd.cups-pdf application/pdf 10 -"', '"application/vnd.cups-postscript application/postscript 10 -"']
    """
    def parse_content(self, content):
        if not content:
            raise SkipException("No Valid Configuration")
        data = {}
        for line in content:
            if line.startswith("*") and ":" in line and not line.startswith("*%"):
                key = line.split(":")[0].split("*")[-1].strip()
                value = line.split(":")[-1].strip()
                if key in data:
                    if isinstance(data[key], list):
                        data[key].append(value)
                    else:
                        data[key] = [data[key], value]
                else:
                    data[key] = value
        if not data:
            raise SkipException("No Valid Configuration")
        self.update(data)



"""
json_parser handles primitive json parsing. It doesn't handle unicode or
numbers in scientific notation.
"""
from insights.parsr import (Colon, Comma, EOF, Forward, Literal, LeftBracket,
        LeftCurly, Number, RightBracket, RightCurly, QuotedString, WS)


def loads(data):
    return Top(data)[0]


def load(f):
    return loads(f.read())


JsonArray = Forward()
JsonObject = Forward()
TRUE = Literal("true", value=True)
FALSE = Literal("false", value=False)
NULL = Literal("null", value=None)
SimpleValue = (Number | QuotedString | JsonObject | JsonArray | TRUE | FALSE | NULL)
JsonValue = (WS >> SimpleValue << WS)
Key = (QuotedString << Colon)
KVPairs = (((WS >> Key) + JsonValue).sep_by(Comma))
JsonArray <= (LeftBracket >> JsonValue.sep_by(Comma) << RightBracket)
JsonObject <= (LeftCurly >> KVPairs.map(lambda res: dict((k, v) for (k, v) in res)) << RightCurly)
Top = JsonValue + EOF



import string
from insights.parsr import InSet, String, DoubleQuotedString, QuotedString


def test_inset():
    assert InSet("abc", "set of abc")("a") == "a"


def test_string():
    sb = String(string.ascii_letters)
    data = "abcde"
    assert sb(data) == "abcde"


def test_quoted_string():
    data = "'abcde'"
    assert QuotedString(data) == "abcde"


def test_escaped_string():
    data = r"""
    "%h %l %u %t \"%r\" %>s %b \"%{Referer}i\" \"%{User-Agent}i\""
    """.strip()
    assert DoubleQuotedString(data)



from insights.parsers.sysconfig import CorosyncSysconfig
from insights.tests import context_wrap

corosync_content = """
# Corosync init script configuration file

# COROSYNC_INIT_TIMEOUT specifies number of seconds to wait for corosync
# initialization (default is one minute).
COROSYNC_INIT_TIMEOUT=60

# COROSYNC_OPTIONS specifies options passed to corosync command
# (default is no options).
# See "man corosync" for detailed descriptions of the options.
COROSYNC_OPTIONS=""
"""


def test_corosync_sysconfig():
    result = CorosyncSysconfig(context_wrap(corosync_content))
    assert result.data['COROSYNC_OPTIONS'] == ""
    assert result.data['COROSYNC_INIT_TIMEOUT'] == "60"

    assert result.options == ''
    assert result.unparsed_lines == []



import pytest
import doctest

from insights.parsers import package_provides_httpd
from insights.parsers.package_provides_httpd import PackageProvidesHttpd
from insights.tests import context_wrap
from ...parsers import SkipException


PACKAGE_COMMAND_MATCH = """
/opt/rh/httpd24/root/usr/sbin/httpd httpd24-httpd-2.4.34-7.el7.x86_64
"""

PACKAGE_COMMAND_ERROR = """
"""

PACKAGE_COMMAND_NOT_MATCH = """
bin/httpd file /root/bin/httpd is not owned by any package
"""


def test_package_provides_httpd_match():
    package = PackageProvidesHttpd(context_wrap(PACKAGE_COMMAND_MATCH))
    assert package.command == "/opt/rh/httpd24/root/usr/sbin/httpd"
    assert package.package == "httpd24-httpd-2.4.34-7.el7.x86_64"


def test_package_provides_httpd_err():
    with pytest.raises(SkipException) as pe:
        PackageProvidesHttpd(context_wrap(PACKAGE_COMMAND_ERROR))
        assert "there is not httpd application running" in str(pe)


def test_package_provides_httpd_not_match():
    with pytest.raises(SkipException) as pe:
        PackageProvidesHttpd(context_wrap(PACKAGE_COMMAND_NOT_MATCH))
        assert "current running httpd command is not provided by package installed through yum or rpm" in str(pe)


def test_doc_examples():
    env = {
        'package': package_provides_httpd.PackageProvidesHttpd(context_wrap(PACKAGE_COMMAND_MATCH)),
    }
    failed, _ = doctest.testmod(package_provides_httpd, globs=env)
    assert failed == 0



from insights.parsers.insights_client_conf import InsightsClientConf
from insights.tests import context_wrap

CLIENT_CONF = """
[insights-client]
auto_update=False
"""


def test_insights_client_conf():
    conf = InsightsClientConf(context_wrap(CLIENT_CONF))
    assert conf is not None
    assert list(conf.sections()) == ['insights-client']
    assert conf.has_option('insights-client', 'auto_update')
    assert not conf.has_option('yabba', 'dabba_do')
    assert conf.get('insights-client', 'auto_update') == 'False'



import doctest

from insights.parsers import virt_uuid_facts
from insights.parsers.tests import skip_exception_check
from insights.parsers.virt_uuid_facts import VirtUuidFacts
from insights.tests import context_wrap


VIRT_UUID_FACTS = """
{"virt.uuid": "4546B285-6C41-5D6R-86G5-0BFR4B3625FS", "uname.machine": "x86"}
""".strip()


def test_virt_uuid_facts():
    result = VirtUuidFacts(context_wrap(VIRT_UUID_FACTS))

    assert result.data == {
            'virt.uuid': '4546B285-6C41-5D6R-86G5-0BFR4B3625FS',
            "uname.machine": "x86"
    }
    assert result.data['virt.uuid'] == '4546B285-6C41-5D6R-86G5-0BFR4B3625FS'


def test_virt_uuid_facts_empty():
    assert 'Empty output.' in skip_exception_check(VirtUuidFacts)


def test_virt_uuid_facts_doc_examples():
    env = {
        'VirtUuidFacts': VirtUuidFacts,
        'virt_uuid_facts': VirtUuidFacts(context_wrap(VIRT_UUID_FACTS)),
    }
    failed, total = doctest.testmod(virt_uuid_facts, globs=env)
    assert failed == 0



from insights.parsers import getenforce
from insights.tests import context_wrap

GETENFORCE1 = "Enforcing"
GETENFORCE2 = "Permissive"
GETENFORCE3 = "Disabled"


class Testgetenforce():
    def test_getenforce(self):
        result = getenforce.getenforcevalue(context_wrap(GETENFORCE1))
        assert result.get("status") == 'Enforcing'

        result = getenforce.getenforcevalue(context_wrap(GETENFORCE2))
        assert result.get("status") == 'Permissive'

        result = getenforce.getenforcevalue(context_wrap(GETENFORCE3))
        assert result.get("status") == 'Disabled'



from .. import Parser, parser, get_active_lines, LegacyItemAccess, IniConfigFile
from insights.specs import Specs


@parser(Specs.pluginconf_d)
class PluginConfD(LegacyItemAccess, Parser):
    """Class to parse configuration file under ``pluginconf.d``

    Attributes:
        data (dict): A dict likes
            {
                "main": {
                    "gpgcheck": "1",
                    "enabled": "0",
                    "timeout": "120"
                }
            }
    """

    def parse_content(self, content):
        """
        ---Sample---
        [main]
        enabled = 0
        gpgcheck = 1
        timeout = 120

        # You can specify options per channel, e.g.:
        #
        #[rhel-i386-server-5]
        #enabled = 1
        #
        #[some-unsigned-custom-channel]
        #gpgcheck = 0
        """
        plugin_dict = {}
        section_dict = {}
        key = None
        for line in get_active_lines(content):
            if line.startswith('['):
                section_dict = {}
                plugin_dict[line[1:-1]] = section_dict
            elif '=' in line:
                key, _, value = line.partition("=")
                key = key.strip()
                section_dict[key] = value.strip()
            else:
                if key:
                    section_dict[key] = ','.join([section_dict[key], line])
        self.data = plugin_dict

    def __iter__(self):
        for sec in self.data:
            yield sec


@parser(Specs.pluginconf_d)
class PluginConfDIni(IniConfigFile):
    """
    Read yum plugin config files, in INI format, using the standard INI file
    parser class.
    """
    pass



from insights.parsers.iscsiadm_mode_session import IscsiAdmModeSession
from insights.tests import context_wrap

ISCSIADM_SESSION_INFO = """
tcp: [1] 10.72.32.45:3260,1 iqn.2017-06.com.example:server1 (non-flash)
tcp: [2] 10.72.32.45:3260,1 iqn.2017-06.com.example:server2 (non-flash)
""".strip()


EXPECTED_RESULTS = [{'IFACE_TRANSPORT': "tcp",
                      'SID': '1',
                      'TARGET_IP': '10.72.32.45:3260,1',
                      'TARGET_IQN': 'iqn.2017-06.com.example:server1'},
                     {'IFACE_TRANSPORT': "tcp",
                      'SID': '2',
                      'TARGET_IP': '10.72.32.45:3260,1',
                      'TARGET_IQN': 'iqn.2017-06.com.example:server2'}]


def test_iscsiadm_session_info():
    iscsiadm_session_info = IscsiAdmModeSession(context_wrap(ISCSIADM_SESSION_INFO))

    assert iscsiadm_session_info.data == EXPECTED_RESULTS
    assert len(iscsiadm_session_info.data) == 2
    assert iscsiadm_session_info[0] == {
        'IFACE_TRANSPORT': 'tcp',
        'SID': '1',
        'TARGET_IP': '10.72.32.45:3260,1',
        'TARGET_IQN': 'iqn.2017-06.com.example:server1'
    }
    assert iscsiadm_session_info[1]['TARGET_IQN'] == 'iqn.2017-06.com.example:server2'



"""
sapcontrol - Commands ``sapcontrol``
====================================
Shared parsers for parsing output of the ``sapcontrol [option]`` commands.

SAPControlSystemUpdateList- command ``sapcontrol -nr <NR> -function GetSystemUpdateList``
-----------------------------------------------------------------------------------------
"""
from insights import parser, CommandParser
from insights.parsers import SkipException, ParseException, parse_delimited_table
from insights.specs import Specs


@parser(Specs.sapcontrol_getsystemupdatelist)
class SAPControlSystemUpdateList(CommandParser):
    """
    This class provides processing for the output of the
    ``sapcontrol -nr <NR> -function GetSystemUpdateList`` command on SAP system.

    Sample output of the command::

        29.01.2019 01:20:36
        GetSystemUpdateList
        OK
        hostname, instanceNr, status, starttime, endtime, dispstatus
        vm37-39, 00, Running, 29.01.2019 00:00:02, 29.01.2019 01:10:11, GREEN
        vm37-39, 02, Running, 29.01.2019 00:00:05, 29.01.2019 01:11:11, GREEN
        vm37-39, 03, Running, 29.01.2019 00:00:05, 29.01.2019 01:12:36, GREEN

    Examples:
        >>> rks.is_running
        True
        >>> rks.is_green
        True
        >>> rks.data[-1]['status'] == 'Running'
        True
        >>> rks.data[-1]['dispstatus'] == 'GREEN'
        True
        >>> rks.data[0]['instanceNr'] == '00'
        True

    Attributes:
        is_running (Boolean): The status of GetSystemUpdateList
        is_green (Boolean): The display status of GetSystemUpdateList
        data (list): List of dicts where keys are the lead name of header line and
            values are the string value.
    """
    def parse_content(self, content):
        if not content:
            raise SkipException("Empty output.")

        header = "hostname, instanceNr, status, starttime, endtime, dispstatus"
        if len(content) <= 3 or header not in content:
            raise ParseException("Incorrect content: '{0}'".format(content[-1]))

        header_sp = [i.strip() for i in header.split(',')]
        self.data = parse_delimited_table(
                content,
                delim=',',
                max_splits=5,
                strip=True,
                heading_ignore=header_sp)

        if not self.data:
            raise SkipException("Empty or useless output.")

        self.is_running = all(l['status'] == 'Running' for l in self.data)
        self.is_green = all(l['dispstatus'] == 'GREEN' for l in self.data)



import doctest
import pytest

from insights.parsers import dig, SkipException
from insights.parsers.dig import Dig, DigDnssec, DigEdns, DigNoedns
from insights.tests import context_wrap

SIGNED_DNSSEC = """; <<>> DiG 9.11.1-P3-RedHat-9.11.1-2.P3.fc26 <<>> +dnssec nic.cz. SOA
;; global options: +cmd
;; Got answer:
;; ->>HEADER<<- opcode: QUERY, status: NOERROR, id: 58794
;; flags: qr rd ra; QUERY: 1, ANSWER: 2, AUTHORITY: 0, ADDITIONAL: 1

;; OPT PSEUDOSECTION:
; EDNS: version: 0, flags: do; udp: 4096
;; QUESTION SECTION:
;nic.cz.                                IN      SOA

;; ANSWER SECTION:
nic.cz.                 278     IN      SOA     a.ns.nic.cz.
hostmaster.nic.cz. 1508686803 10800 3600 1209600 7200
nic.cz.                 278     IN      RRSIG   SOA 13 2 1800
20171105143612 20171022144003 41758 nic.cz.
hq3rr8dASRlucMJxu2QZnX6MVaMYsKhmGGxBOwpkeUrGjfo6clzG6MZN
2Jy78fWYC/uwyIsI3nZMUKv573eCWg==

;; Query time: 22 msec
;; SERVER: 10.38.5.26#53(10.38.5.26)
;; WHEN: Tue Oct 24 14:28:56 CEST 2017
;; MSG SIZE  rcvd: 189"""

NOT_SIGNED_DNSSEC = """; <<>> DiG 9.11.1-P3-RedHat-9.11.1-2.P3.fc26 <<>> +dnssec google.com. SOA
;; global options: +cmd
;; Got answer:
;; ->>HEADER<<- opcode: QUERY, status: NOERROR, id: 13253
;; flags: qr rd ra; QUERY: 1, ANSWER: 1, AUTHORITY: 0, ADDITIONAL: 1

;; OPT PSEUDOSECTION:
; EDNS: version: 0, flags: do; udp: 4096
;; QUESTION SECTION:
;google.com.                    IN      SOA

;; ANSWER SECTION:
google.com.             60      IN      SOA     ns1.google.com.
dns-admin.google.com. 173219439 900 900 1800 60

;; Query time: 46 msec
;; SERVER: 10.38.5.26#53(10.38.5.26)
;; WHEN: Tue Oct 24 14:28:20 CEST 2017
;; MSG SIZE  rcvd: 89"""

BAD_DNSSEC = """; <<>> DiG 9.11.1-P3-RedHat-9.11.1-2.P3.fc26 <<>> +dnssec google.com. SOA
;; global options: +cmd
;; Got answer:
;; ->>HEADER<<- opcode: QUERY, status: REFUSED, id: 13253
;; flags: qr rd ra; QUERY: 1, ANSWER: 1, AUTHORITY: 0, ADDITIONAL: 1

;; OPT PSEUDOSECTION:
; EDNS: version: 0, flags: do; udp: 4096
;; QUESTION SECTION:
;google.com.                    IN      SOA

;; ANSWER SECTION:
google.com.             60      IN      SOA     ns1.google.com.
dns-admin.google.com. 173219439 900 900 1800 60

;; Query time: 46 msec
;; SERVER: 10.38.5.26#53(10.38.5.26)
;; WHEN: Tue Oct 24 14:28:20 CEST 2017
;; MSG SIZE  rcvd: 89"""

GOOD_EDNS = """; <<>> DiG 9.11.1-P3-RedHat-9.11.1-3.P3.fc26 <<>> +edns=0 . SOA
;; global options: +cmd
;; Got answer:
;; ->>HEADER<<- opcode: QUERY, status: NOERROR, id: 11158
;; flags: qr rd ra ad; QUERY: 1, ANSWER: 1, AUTHORITY: 0, ADDITIONAL: 1

;; OPT PSEUDOSECTION:
; EDNS: version: 0, flags:; udp: 4096
;; QUESTION SECTION:
;.				IN	SOA

;; ANSWER SECTION:
.			19766	IN	SOA	a.root-servers.net. nstld.verisign-grs.com. 2017120600 1800 900 604800 86400

;; Query time: 22 msec
;; SERVER: 10.38.5.26#53(10.38.5.26)
;; WHEN: Thu Dec 07 09:38:33 CET 2017
;; MSG SIZE  rcvd: 103"""

BAD_EDNS = """; <<>> DiG 9.11.1-P3-RedHat-9.11.1-3.P3.fc26 <<>> +edns=0 . SOA
;; global options: +cmd
;; Got answer:
;; ->>HEADER<<- opcode: QUERY, status: SERVFAIL, id: 11158
;; flags: qr rd ra ad; QUERY: 1, ANSWER: 1, AUTHORITY: 0, ADDITIONAL: 1

;; OPT PSEUDOSECTION:
; EDNS: version: 0, flags:; udp: 4096
;; QUESTION SECTION:
;.				IN	SOA

;; ANSWER SECTION:
.			19766	IN	SOA	a.root-servers.net. nstld.verisign-grs.com. 2017120600 1800 900 604800 86400

;; Query time: 22 msec
;; SERVER: 10.38.5.26#53(10.38.5.26)
;; WHEN: Thu Dec 07 09:38:33 CET 2017
;; MSG SIZE  rcvd: 103"""

GOOD_NOEDNS = """; <<>> DiG 9.11.1-P3-RedHat-9.11.1-3.P3.fc26 <<>> +noedns . SOA
;; global options: +cmd
;; Got answer:
;; ->>HEADER<<- opcode: QUERY, status: NOERROR, id: 47135
;; flags: qr rd ra ad; QUERY: 1, ANSWER: 1, AUTHORITY: 0, ADDITIONAL: 0

;; QUESTION SECTION:
;.				IN	SOA

;; ANSWER SECTION:
.			20195	IN	SOA	a.root-servers.net. nstld.verisign-grs.com. 2017120600 1800 900 604800 86400

;; Query time: 22 msec
;; SERVER: 10.38.5.26#53(10.38.5.26)
;; WHEN: Thu Dec 07 09:31:24 CET 2017
;; MSG SIZE  rcvd: 92"""

BAD_NOEDNS = """; <<>> DiG 9.11.1-P3-RedHat-9.11.1-2.P3.fc26 <<>> +noedns
ewf-dwqfwqf-gdsa.com SOA
;; global options: +cmd
;; Got answer:
;; ->>HEADER<<- opcode: QUERY, status: NXDOMAIN, id: 30634
;; flags: qr rd ra; QUERY: 1, ANSWER: 0, AUTHORITY: 1, ADDITIONAL: 0

;; QUESTION SECTION:
;ewf-dwqfwqf-gdsa.com.          IN      SOA

;; AUTHORITY SECTION:
com.                    900     IN      SOA     a.gtld-servers.net.
nstld.verisign-grs.com. 1508851057 1800 900 604800 86400

;; Query time: 29 msec
;; SERVER: 10.38.5.26#53(10.38.5.26)
;; WHEN: Tue Oct 24 15:17:53 CEST 2017
;; MSG SIZE  rcvd: 111"""


def test_dig_no_data():
    with pytest.raises(SkipException):
        Dig(context_wrap(""), "")


def test_dig_dnssec():
    dig_dnssec = DigDnssec(context_wrap(SIGNED_DNSSEC))
    assert dig_dnssec.status == "NOERROR"
    assert dig_dnssec.has_signature

    dig_dnssec = DigDnssec(context_wrap(NOT_SIGNED_DNSSEC))
    assert dig_dnssec.status == "NOERROR"
    assert not dig_dnssec.has_signature

    dig_dnssec = DigDnssec(context_wrap(BAD_DNSSEC))
    assert dig_dnssec.status == "REFUSED"
    assert not dig_dnssec.has_signature


def test_dig_edns():
    dig_edns = DigEdns(context_wrap(GOOD_EDNS))
    assert dig_edns.status == "NOERROR"
    assert not dig_edns.has_signature

    dig_edns = DigEdns(context_wrap(BAD_EDNS))
    assert dig_edns.status == "SERVFAIL"
    assert not dig_edns.has_signature


def test_dig_noedns():
    dig_noedns = DigNoedns(context_wrap(GOOD_NOEDNS))
    assert dig_noedns.status == "NOERROR"
    assert not dig_noedns.has_signature

    dig_noedns = DigNoedns(context_wrap(BAD_NOEDNS))
    assert dig_noedns.status == "NXDOMAIN"
    assert not dig_noedns.has_signature


def test_doc_examples():
    env = {
        "dig_dnssec": DigDnssec(context_wrap(SIGNED_DNSSEC)),
        "dig_edns": DigEdns(context_wrap(GOOD_EDNS)),
        "dig_noedns": DigNoedns(context_wrap(GOOD_NOEDNS)),
    }
    failed, total = doctest.testmod(dig, globs=env)
    assert failed == 0



"""
SockStats - file ``/proc/net/sockstat``
=======================================
The ``TcpIpStats`` class implements the parsing of ``/proc/net/sockstat``
file, which contains TCP/IP stats of individual layer.
"""


from insights import Parser, parser
from insights.parsers import SkipException
from insights.specs import Specs


@parser(Specs.sockstat)
class SockStats(Parser, dict):
    """
    Parser for ``/proc/net/sockstat`` file.

    Sample input is provided in the *Examples*.

    Sample content::

        sockets: used 3037
        TCP: inuse 1365 orphan 17 tw 2030 alloc 2788 mem 4109
        UDP: inuse 6 mem 3
        UDPLITE: inuse 0
        RAW: inuse 0
        FRAG: inuse 0 memory 0

    Examples:

        >>> type(sock_obj)
        <class 'insights.parsers.sockstat.SockStats'>
        >>> sock_obj.seg_details('tcp')['mem']
        '4109'
        >>> sock_obj.seg_element_details('tcp', 'mem')
        4109
        >>> sock_obj.seg_element_details('frag', 'inuse')
        0
        >>> sock_obj.get('sockets')
        {'used': '3037'}
        >>> sock_obj.get('sockets').get('used')
        '3037'
        >>> sock_obj.seg_element_details('tcp', 'abc') is None
        True


    Resultant Data::

        {
            'sockets':
                    {
                        'used': '3037'
                    },
            'tcp':
                 {
                     'inuse': '1365',
                     'orphan': '17',
                     'tw': '2030',
                     'alloc': '2788',
                     'mem': '4109'
                 },
            'udp':
                 {
                     'inuse': '6',
                     'mem': '3'
                 },
            'udplite':
                     {
                        'inuse': '0'
                     },
            'raw':
                 {
                    'inuse': '0'
                 }
            'frag':
                  {
                    'inuse': '0',
                    'memory': '0'
                  }
        }

    Raises:
        SkipException: When contents are empty
    """

    def parse_content(self, content):

        if not content:
            raise SkipException("No Contents")

        for line in content:
            line_split = line.split(':')
            nt_seg = line_split[0].lower()
            self[nt_seg] = {}
            val_lst = line_split[1].strip().split()
            key = True
            # Convert string to dictionary
            for idx in val_lst:
                if key:
                    key_idx = idx.lower()
                    self[nt_seg][key_idx] = None
                    key = False
                else:
                    self[nt_seg][key_idx] = idx
                    key = True

    @property
    def sock_stats(self):
        """
        Returns (dict): On On success, it will return detailed memory consumption
        done by all TCP/IP layer in single data, on failure it will return ``None``
        """
        return self

    def seg_details(self, seg):
        """
        Returns (dict): On success, it will return detailed memory consumption
        done by each segment(TCP/IP layer), on failure it will return ``None``.
        """
        return self.get(seg, None)

    def seg_element_details(self, seg, elem):
        """
        Returns (int): On success, it will return memory consumption done by each
        element of the segment(TCP/IP layer), on failure it will return ``None``.
        """

        if seg and elem and elem in self.get(seg, {}):
            return int(self.get(seg, {}).get(elem))
        return None



"""
PuppetCertExpireDate - command ``openssl x509 -in /etc/puppetlabs/puppet/ssl/ca/ca_crt.pem -enddate -noout``
============================================================================================================

The PuppetCertExpireDate parser reads the output of
``openssl x509 -in /etc/puppetlabs/puppet/ssl/ca/ca_crt.pem -enddate -noout``.

Sample output of ``openssl x509 -in /etc/puppetlabs/puppet/ssl/ca/ca_crt.pem -enddate -noout``::

    notAfter=Dec  4 07:04:05 2035 GMT

Examples::

    >>> type(date_info)
    <class 'insights.parsers.puppet_ca_cert_expire_date.PuppetCertExpireDate'>
    >>> date_info['notAfter'].datetime
    datetime.datetime(2035, 12, 4, 7, 4, 5)

"""

from insights import parser
from insights.specs import Specs
from insights.parsers import SkipException
from insights.parsers.ssl_certificate import CertificateInfo


@parser(Specs.puppet_ca_cert_expire_date)
class PuppetCertExpireDate(CertificateInfo):
    """
    .. note::
        Please refer to its super-class :class:`insights.parsers.ssl_certificate.CertificateInfo` for more
        details.

    .. warning::
        The attribute expire_date is deprecated, please get the value from the dictionary directly instead.

    Read the ``openssl x509 -in /etc/puppetlabs/puppet/ssl/ca/ca_crt.pem -enddate -noout``
    and set the date to property ``expire_date``.

    Attributes:
        expire_date (datetime): The date when the puppet ca cert will be expired

    Raises:
        SkipException: when notAfter isn't in the output
    """

    def parse_content(self, content):
        super(PuppetCertExpireDate, self).parse_content(content)
        if 'notAfter' not in self:
            raise SkipException("Cannot get the puppet ca cert expire info")
        self.expire_date = self['notAfter'].datetime



from insights.parsers.resolv_conf import ResolvConf
from insights.tests import context_wrap


RESOLVCONF = '''
# This file is being maintained by Puppet.
# DO NOT EDIT
search a.b.com b.c.com
options timeout:2 attempts:2
nameserver 10.160.224.51
nameserver 10.160.225.51
nameserver 10.61.193.11
'''

RESOLVCONF_M = '''
; Generated by NetworkManager
search ttt.com
domain ttt.com
nameserver 192.168.30.1
'''

RESOLV_CONF_NO_ACTIVE = '''
nameserver 192.168.30.1
'''


def test_resolv_conf():
    resolv_info = ResolvConf(context_wrap(RESOLVCONF))

    assert resolv_info.get('search') == ['a.b.com', 'b.c.com']
    assert resolv_info.get('options') == ['timeout:2', 'attempts:2']
    assert resolv_info.get('nameserver') == ['10.160.224.51', '10.160.225.51', '10.61.193.11']
    assert resolv_info.get('active') == 'search'


# Testing when 'search' and 'domain' keywords both exist.
def test_resolv_conf_m():
    resolv_info = ResolvConf(context_wrap(RESOLVCONF_M))

    assert resolv_info.get('domain') == ['ttt.com']
    assert resolv_info.get('search') == ['ttt.com']
    assert resolv_info.get('nameserver') == ['192.168.30.1']
    assert resolv_info.get('active') == 'domain'


# And testing when neither 'search' nor 'domain' keywords are found.
def test_resolv_conf_neither():
    resolv = ResolvConf(context_wrap(RESOLV_CONF_NO_ACTIVE))

    assert resolv['nameserver'] == ['192.168.30.1']
    assert resolv['active'] == ''



"""
Dcbtool - Command ``/sbin/dcbtool gc {interface} dcb``
======================================================

    Parse Lines from the `dcbtool gc eth1 dcb` to check DCBX if enabled

    Successful completion of the command returns data similar to::

        Command:    Get Config
        Feature:    DCB State
        Port:       eth0
        Status:     Off
        DCBX Version: FORCED CIN

    The keys in this data are converted to lower case and spaces are converted
    to underscores.

    An `is_on` attribute is also provided to indicate if the status is 'On'.

    Examples:

        >>> dcbstate = shared[Dcbtool]
        >>> dcb.data
        {
            "command": "Get Config",
            "feature": "DCB State",
            "port": "eth0",
            "status": "Off"
            "dcbx_version":"FORCED CIN"
        }
        >>> dcb['port']
        'eth0'
        >>> dcb['state']
        'Off'
        >>> dcb.is_on
        False

    If a "Connection refused" error is encountered,
    an empty dictionary is returned`.

"""

from .. import parser, LegacyItemAccess, CommandParser, get_active_lines
from insights.specs import Specs


@parser(Specs.dcbtool_gc_dcb)
class Dcbtool(LegacyItemAccess, CommandParser):
    """
    Parse the output of the `dcbtool` command.

    If the command output contains 'Connection refused', no data is stored.
    The LegacyItemAccess mixin class is used to provide direct access to the
    data.

    Attributes:
        data (dict): A dictionary of the content of the command output.
        is_on: (bool): Is the status of the interface 'On'?
    """
    def parse_content(self, content):
        self.data = {}
        if "Connection refused" in content[0]:
            return

        for line in get_active_lines(content):
            key, value = line.split(":", 1)
            key = key.lower().replace(" ", "_")
            self.data[key] = value.strip()

        self.is_on = (self.data['status'] == 'On')



"""
Bash Version
============

This is a simple rule and can be run against the local host
using the following command::

$ insights-run -p examples.rules.bash_version

or from the examples/rules directory::

$ python sample_rules.py
"""
from insights.core.plugins import make_info, rule
from insights.parsers.installed_rpms import InstalledRpms

KEY = "BASH_VERSION"

CONTENT = "Bash RPM Version: {{ bash_version }}"


@rule(InstalledRpms)
def report(rpms):
    bash = rpms.get_max('bash')
    return make_info(KEY, bash_version=bash.nvra)



from insights.core import JSONParser
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.tags)
class Tags(JSONParser):
    """ Class for parsing the content of ``tags.json``."""
    pass



"""
MistralExecutorLog - file ``/var/log/mistral/executor.log``
===========================================================
"""

from insights import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.mistral_executor_log)
class MistralExecutorLog(LogFileOutput):
    """Parse the ``/var/log/mistral/executor.log`` file.

    Provide access to mistral executor log using the LogFileOutput parser class.

    Typical content of ``executor.log`` file is::

        2019-03-01 13:54:41.091 26749 DEBUG oslo_concurrency.lockutils [-] Acquired semaphore "singleton_lock" lock /usr/lib/python2.7/site-packages/oslo_concurrency/lockutils.py:212
        2019-03-01 13:54:41.091 26749 DEBUG oslo_concurrency.lockutils [-] Releasing semaphore "singleton_lock" lock /usr/lib/python2.7/site-packages/oslo_concurrency/lockutils.py:228
        2019-03-01 13:54:41.092 26749 DEBUG oslo_service.service [-] Full set of CONF: _wait_for_exit_or_signal /usr/lib/python2.7/site-packages/oslo_service/service.py:303
        2019-03-01 13:54:41.133 26749 DEBUG oslo_concurrency.lockutils [-] Lock "service_coordinator" released by "mistral.service.coordination.register_membership" :: held 0.000s inner /usr/lib/python2.7/site-packages/oslo_concurrency/lockutils.py:285
        2019-03-01 13:56:05.329 26749 INFO mistral.executors.executor_server [req-a1d40531-b3b9-4fdf-802f-0371bf364551 89665ff87df144128ada2b8c59ec62f1 e4d0868a0715411fa1481f8ccfb61414 - default default] Received RPC request 'run_action'[action_ex_id=7c2c79ba-d7c4-4c4d-9e51-684bab00e6a9, action_cls_str=mistral.actions.std_actions.NoOpAction, action_cls_attrs={}, params={}, timeout=None]
        2019-03-01 14:36:55.134 26749 ERROR mistral.executors.default_executor ActionException: HeatAction.stacks.get failed: ERROR: The Stack (overcloud) could not be found.

    Examples:
        >>> type(executor_log)
        <class 'insights.parsers.mistral_log.MistralExecutorLog'>
        >>> executor_log.get('mistral.executors.default_executor')[0].get('raw_message')
        '2019-03-01 14:36:55.134 26749 ERROR mistral.executors.default_executor ActionException: HeatAction.stacks.get failed: ERROR: The Stack (overcloud) could not be found.'
        >>> from datetime import datetime
        >>> len(list(executor_log.get_after(datetime(2019, 3, 1, 13, 56, 0))))
        2

    """
    pass



import pytest
import doctest

from insights.parsers import gcp_license_codes
from insights.parsers.gcp_license_codes import GCPLicenseCodes
from insights.tests import context_wrap
from insights.parsers import SkipException, ParseException

GCP_LICENSE_CODES_1 = '[{"id": "123451234512345"}]'
GCP_LICENSE_CODES_2 = '[{"id": "123451234512345"}, {"id": "238949287234"}]'
GCP_LICENSE_CODES_3 = '[]'

GCP_LICENSE_CODES_4 = """
 % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                  Dload  Upload   Total   Spent    Left  Speed
   0     0    0     0    0     0      0      0 --:--:-- --:--:-- --:--:--     0
 100  1126  100  1126    0     0  1374k      0 --:--:-- --:--:-- --:--:-- 1099k
[{"id": "123451234512345"}]
"""

GCP_LICENSE_CODES_DOC = '[{"id": "601259152637613565"}]'

GCP_LICENSE_CODES_AB_1 = """
curl: (7) Failed to connect to 169.254.169.254 port 80: Connection timed out
""".strip()
GCP_LICENSE_CODES_AB_2 = """
curl: (7) couldn't connect to host
""".strip()
GCP_LICENSE_CODES_AB_3 = """
curl: (28) connect() timed out!
""".strip()


def test_azure_instance_place_ab_other():
    with pytest.raises(SkipException):
        GCPLicenseCodes(context_wrap(GCP_LICENSE_CODES_AB_1))

    with pytest.raises(SkipException):
        GCPLicenseCodes(context_wrap(GCP_LICENSE_CODES_AB_2))

    with pytest.raises(SkipException):
        GCPLicenseCodes(context_wrap(GCP_LICENSE_CODES_AB_3))

    with pytest.raises(SkipException):
        GCPLicenseCodes(context_wrap(''))

    with pytest.raises(ParseException):
        GCPLicenseCodes(context_wrap(GCP_LICENSE_CODES_4))


def test_gcp_license_codes():
    gcp_licenses = GCPLicenseCodes(context_wrap(GCP_LICENSE_CODES_1))
    assert gcp_licenses.ids == ["123451234512345"]
    assert gcp_licenses.raw == [{"id": "123451234512345"}]

    gcp_licenses = GCPLicenseCodes(context_wrap(GCP_LICENSE_CODES_2))
    assert gcp_licenses.ids == ["123451234512345", "238949287234"]
    assert gcp_licenses.raw == [{"id": "123451234512345"}, {"id": "238949287234"}]

    gcp_licenses = GCPLicenseCodes(context_wrap(GCP_LICENSE_CODES_3))
    assert gcp_licenses.ids is None
    assert gcp_licenses.raw == []


def test_doc_examples():
    env = {
        'gcp_licenses': GCPLicenseCodes(context_wrap(GCP_LICENSE_CODES_DOC))
    }
    failed, total = doctest.testmod(gcp_license_codes, globs=env)
    assert failed == 0



"""
ZdumpV - command ``/usr/sbin/zdump -v /etc/localtime -c 2019,2039``
===================================================================

The ``/usr/sbin/zdump -v /etc/localtime -c 2019,2039`` command provides information about
'Daylight Saving Time' in file /etc/localtime from 2019 to 2039.

Sample content from command ``zdump -v /etc/localtime -c 2019,2039`` is::

    /etc/localtime  Sun Mar 10 06:59:59 2019 UTC = Sun Mar 10 01:59:59 2019 EST isdst=0 gmtoff=-18000
    /etc/localtime  Sun Mar 10 07:00:00 2019 UTC = Sun Mar 10 03:00:00 2019 EDT isdst=1 gmtoff=-14400
    /etc/localtime  Sun Nov  7 05:59:59 2038 UTC = Sun Nov  7 01:59:59 2038 EDT isdst=1 gmtoff=-14400
    /etc/localtime  Sun Nov  7 06:00:00 2038 UTC = Sun Nov  7 01:00:00 2038 EST isdst=0 gmtoff=-18000

Examples:
    >>> dst = zdump[0]
    >>> dst.get('utc_time')
    datetime.datetime(2019, 3, 10, 6, 59, 59)
    >>> dst.get('utc_time_raw')
    'Sun Mar 10 06:59:59 2019 UTC'
    >>> dst.get('local_time')
    datetime.datetime(2019, 3, 10, 1, 59, 59)
    >>> dst.get('local_time_raw')
    'Sun Mar 10 01:59:59 2019 EST'
    >>> dst.get('isdst')
    0
    >>> dst.get('gmtoff')
    -18000
"""

from datetime import datetime
from insights.specs import Specs
from insights.parsers import SkipException
from insights import parser, CommandParser


def str2datetime(timestamp, tz=False):
    """
    This function translates the time stamp into a datetime object.

    Args:
        timestamp (str): the time stamp from command `zdump -v`
        tz (bool): True if it's UTC TimeZone.

    Returns:
        time (datetime): the datetime object about the time stamp
        time_string (str): the formatted time stamp
    """
    time, time_string = None, timestamp.strip()

    # Fixed the problem that the program running this python code doesn't
    # has the corresponding TimeZone where strptime will raise ValueError.
    # So, we skip the `TimeZone`
    time_s = time_string.rsplit(None, 1)[0]
    time_f = "%a %b %d %H:%M:%S %Y"

    if tz:
        # In some version, `zdump` prints 'UT' instead of 'UTC'
        # 'UC' is an invalid TimeZone for function `strptime`
        time_s = time_s + " UTC"
        time_f = "%a %b %d %H:%M:%S %Y %Z"

    try:
        time = datetime.strptime(time_s, time_f)
    except ValueError:
        pass

    return time, time_string


@parser(Specs.zdump_v)
class ZdumpV(CommandParser, list):
    """
    Parse the output from the ``/usr/sbin/zdump -v /etc/localtime -c 2019,2039`` command
    and store the 'Daylight Saving Time' information into a list.

    Raises:
        SkipException: When nothing is parsed.

    .. warning:: The value in key `local_time` doesn't include the TimeZone information
    """
    def parse_content(self, content):
        if not content:
            raise SkipException("No Data from command: /usr/sbin/zdump -v /etc/localtime -c 2019,2039")

        for line in content:
            dst = {}
            if 'isdst' not in line:
                # skip the line that does not include a time stamp
                continue

            utc_time, remains = line.strip('/etc/localtime').split(' = ')
            dst['utc_time'], dst['utc_time_raw'] = str2datetime(utc_time, True)
            if dst['utc_time'] is None:
                continue

            local_time, _ = remains.split("isdst")
            dst['local_time'], dst['local_time_raw'] = str2datetime(local_time)
            if dst['local_time'] is None:
                continue

            isdst = [s.split('=')[1] for s in remains.split() if 'isdst' in s and '=' in s]
            if isdst:
                dst['isdst'] = int(isdst[0])

            gmtoff = [s.split('=')[1] for s in remains.split() if 'gmtoff' in s and '=' in s]
            if gmtoff:
                dst['gmtoff'] = int(gmtoff[0])

            self.append(dst)
